// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/billion/perform.c $
// $Rev: 52494 $
// $Date: 2023-03-01 12:30:14 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1B		(1000L * 1000L * 1000L)
#define	_1M		(1024L * 1024L)



// variables ******************************************************************

static	u32	_buffer[_1M];



//*****************************************************************************
static int _read_data(const args_t* args)
{
	int		errs	= 0;
	long	get;
	long	got;
	long	samples	= 0;

	gsc_label("Reading");

	for (;;)
	{
		get	= (_1B - samples) * 4;

		if (get > (sizeof(_buffer)))
			get	= sizeof(_buffer);

		got	= dsi16wrc_read(args->fd, _buffer, get);

		if (got < 0)
		{
			errs	= 1;
			printf(	"FAIL <---  (dsi16wrc_read error, error %ld)\n", got);
			break;
		}

		if (got != get)
		{
			errs	= 1;
			printf(	"FAIL <---  (total %ld, requested %ld samples, got %ld samples)\n",
					(long) samples,
					(long) get / 4,
					(long) got / 4);
			break;
		}

		samples	+= got / 4;

		if (samples >= _1B)
			break;
	}

	if (errs == 0)
	{
		printf("PASS  (read ");
		gsc_label_long_comma(samples);
		printf(" samples)\n");
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs	= 0;

	errs	+= dsi16wrc_config_ai		(args->fd, -1, 1, -1, 100000);
	errs	+= dsi16wrc_rx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= dsi16wrc_rx_io_overflow	(args->fd, -1, 1, DSI16WRC_IO_OVERFLOW_IGNORE, NULL);
	errs	+= dsi16wrc_rx_io_underflow	(args->fd, -1, 1, DSI16WRC_IO_UNDERFLOW_IGNORE, NULL);
	errs	+= _read_data(args);

	return(errs);
}



