// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/dout/perform.c $
// $Rev: 52496 $
// $Date: 2023-03-01 12:30:47 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	DIO_QTY		4

#ifdef CLOCK_MONOTONIC_RAW
#define	CLOCK		CLOCK_MONOTONIC_RAW
#else
#ifdef CLOCK_MONOTONIC
#define	CLOCK		CLOCK_MONOTONIC
#else
#define	CLOCK		CLOCK_REALTIME
#endif
#endif



//*****************************************************************************
#ifndef	CLOCK_MONOTONIC_RAW
#define	clock_gettime(c,ts)	_my_clock_gettime(ts)

static int _my_clock_gettime(struct timespec* ts)
{
	int				ret;
	struct timeval	tv;

	if (ts)
	{
		ret	= 0;
		gettimeofday(&tv, NULL);
		ts->tv_sec	= tv.tv_sec;
		ts->tv_nsec	= tv.tv_usec * 1000;
	}
	else
	{
		ret	= 1;
	}

	return(ret);
}
#endif



//*****************************************************************************
static int _config(const args_t* args)
{
	int	errs	= 0;

	gsc_label("Configuration");
	printf("\n");
	gsc_label_level_inc();

	errs	+= dsi16wrc_initialize(args->fd, -1, 1);
	errs	+= dsi16wrc_dio_dir_out(args->fd, -1, 1, 0xF, NULL);

	gsc_label_level_dec();
	return(errs);
}




//*****************************************************************************
static void _output_header(void)
{
	printf("\n");
	printf("    Seq   DIO Out                             \n");
	printf("    No.   3210  Time Stamp      Delta         \n");
	printf("    ====  ====  ==============  ==============\n");
}



//*****************************************************************************
static int _dev_dio_out(const args_t* args, u16 value)
{
	static	int				count	= 0;
	static	struct timespec	t0;
	static	struct timespec	ts;
	static	struct timespec	ts_last;

	u32						bit;
	long long				delta;
	int						errs	= 0;
	int						i;

	if (count == 0)
	{
		clock_gettime(CLOCK, &t0);
		clock_gettime(CLOCK, &ts);
	}

	ts_last	= ts;
	clock_gettime(CLOCK, &ts);
	errs	+= dsi16wrc_dio_write(args->fd, -1, 0, value & 0xF, NULL);

	count++;
	printf("    %-4d  ", count);

	for (i = DIO_QTY - 1; i >= 0; i--)
	{
		bit	= 0x1 << i;
		printf("%s", (value & bit) ? "1" : "0");

		if (bit & 0x0110)
			printf(" ");
	}

	if (count == 1)
	{
		t0		= ts;
		ts_last	= ts;
	}

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * t0.tv_sec + t0.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	delta	= (1000000000LL * ts.tv_sec + ts.tv_nsec)
			- (1000000000LL * ts_last.tv_sec + ts_last.tv_nsec);
	printf("  %4lld.%09lld", delta / 1000000000, delta % 1000000000);

	printf("\n");
	fflush(stdout);

	if (args->ms_wait)
		os_sleep_ms(args->ms_wait);

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs;
	int	i;

	errs	= _config(args);

	gsc_label("Digital Output");
	printf("\n");
	gsc_label_level_inc();

	_output_header();

	errs	+= _dev_dio_out(args, 0x000);
	errs	+= _dev_dio_out(args, 0x00F);
	errs	+= _dev_dio_out(args, 0x000);
	errs	+= _dev_dio_out(args, 0x00F);
	errs	+= _dev_dio_out(args, 0x000);

	for (i = 0; i < DIO_QTY; i++)
		errs	+= _dev_dio_out(args, 0x1 << i);

	errs	+= _dev_dio_out(args, 0x000);

	for (i = DIO_QTY - 1; i >= 0; i--)
		errs	+= _dev_dio_out(args, 0x1 << i);

	errs	+= _dev_dio_out(args, 0x000);
	errs	+= _dev_dio_out(args, 0x00F);
	errs	+= _dev_dio_out(args, 0x000);
	errs	+= _dev_dio_out(args, 0x00F);
	errs	+= _dev_dio_out(args, 0x000);

	gsc_label_level_inc();

	printf("\n");
	gsc_label("Status");
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	gsc_label_level_dec();
	return(errs);
}



