// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/mbsync/analyze.c $
// $Rev: 52507 $
// $Date: 2023-03-01 12:39:11 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	CHAN_CYCLES_MIN		20
#define	CHAN_VALS_MIN		2000	// 100 values per cycle at 20 cycles
#define	CYCLE_WIDTH_MIN		20
#define	WINDOW_SIZE			5



//*****************************************************************************
// Find the highest value and its location in a window.
static int _locate_high_in_window(int device, int channel, int start)
{
	channel_t*	chan;
	device_t*	dev;
	int			stop	= start + WINDOW_SIZE - 1;
	int			v;
	int			v_high	= -1;
	int			x;
	int			x_high	= -1;

	dev		= &global.device[device];
	chan	= &dev->channel[channel];

	for (x = start; (x <= stop) && (x < chan->value_qty); x++)
	{
		v	= chan->data[x] & 0xFFFF;

		if ((v_high < 0) || (v_high <= v))
		{
			v_high	= v;
			x_high	= x;
		}
	}

	return(x_high);
}



//*****************************************************************************
// Find the highest value and its location in a moving window. We stop when the
// new highest value in the window is lower than the last highest value.
static int _locate_high_in_moving_window(int device, int channel, int start)
{
	channel_t*	chan;
	device_t*	dev;
	int			i;
	int			v;
	int			v_high	= -1;
	int			x;
	int			x_high	= -1;

	dev		= &global.device[device];
	chan	= &dev->channel[channel];

	for (i = start;; i++)
	{
		x	= _locate_high_in_window(device, channel, i);

		if (x < 0)
			break;

		v	= chan->data[x] & 0xFFFF;

		if ((v_high < 0) || (v_high <= v))
		{
			v_high	= v;
			x_high	= x;
		}

		if (i > x_high)
			break;
	}

	return(x_high);
}



//*****************************************************************************
// Find the lowest value and its location in a window.
static int _locate_low_in_window(int device, int channel, int start)
{
	channel_t*	chan;
	device_t*	dev;
	int			stop	= start + WINDOW_SIZE - 1;
	int			v;
	int			v_low	= -1;
	int			x;
	int			x_low	= -1;

	dev		= &global.device[device];
	chan	= &dev->channel[channel];

	for (x = start; (x <= stop) && (x < chan->value_qty); x++)
	{
		v	= chan->data[x] & 0xFFFF;

		if ((v_low < 0) || (v_low >= v))
		{
			v_low	= v;
			x_low	= x;
		}
	}

	return(x_low);
}



//*****************************************************************************
// Find the lowest value and its location in a moving window. We stop when the
// new lowest value in the window is higher than the last lowest value.
static int _locate_low_in_moving_window(int device, int channel, int start)
{
	channel_t*	chan;
	device_t*	dev;
	int			i;
	int			v;
	int			v_low	= -1;
	int			x;
	int			x_low	= -1;

	dev		= &global.device[device];
	chan	= &dev->channel[channel];

	for (i = start;; i++)
	{
		x	= _locate_low_in_window(device, channel, i);

		if (x < 0)
			break;

		v	= chan->data[x] & 0xFFFF;

		if ((v_low < 0) || (v_low >= v))
		{
			v_low	= v;
			x_low	= x;
		}

		if (i > x_low)
			break;
	}

	return(x_low);
}



//*****************************************************************************
// Find the index of the channel's next value that is < 0x800 (zero).
static int _locate_negative(int device, int channel, int next)
{
	channel_t*	chan;
	device_t*	dev;
	int			i;
	int			value;
	int			x		= -1;

	dev		= &global.device[device];
	chan	= &dev->channel[channel];

	for (i = next; i < chan->value_qty; i++)
	{
		value	= chan->data[i] & 0xFFFF;

		if (value < 0x8000)
		{
			x	= i;
			break;
		}
	}

	return(x);
}



//*****************************************************************************
// Find the index of the channel's next value that is >= zero (0x8000).
static int _locate_non_negative(int device, int channel, int next)
{
	channel_t*	chan;
	device_t*	dev;
	int			i;
	int			value;
	int			x		= -1;

	dev		= &global.device[device];
	chan	= &dev->channel[channel];

	for (i = next; i < chan->value_qty; i++)
	{
		value	= chan->data[i] & 0xFFFF;

		if (value >= 0x8000)
		{
			x	= i;
			break;
		}
	}

	return(x);
}



//*****************************************************************************
// Find all LOW, ZERO and HIGH point series for the input sine wave.
static int _find_lzh_points(int device, int channel, int* next, int* low, int* zero, int* high)
{
	int	found	= 0;
	int	x		= next[0];
	int	x1;
	int	x2;
	int	x3;

	low[0]	= -1;
	zero[0]	= -1;
	high[0]	= -1;

	for (;;)
	{
		x	= _locate_non_negative(device, channel, x);

		if (x < 0)
			break;

		x	= _locate_negative(device, channel, x);

		if (x < 0)
			break;

		x1	= _locate_low_in_moving_window(device, channel, x);

		if (x1 < 0)
			break;

		x2	= _locate_non_negative(device, channel, x1);

		if (x2 < 0)
			break;

		x3	= _locate_high_in_moving_window(device, channel, x2);

		if (x3 < 0)
			break;

		found	= 1;
		next[0]	= x;
		low[0]	= x1;
		zero[0]	= x2;
		high[0]	= x3;
		break;
	}

	return(found);
}



//*****************************************************************************
// Verify thet the channel tags are all valid.
int analyze_channel_tags(const args_t* args, int index, char* dst)
{
	int			chan		= 0;
	device_t*	dev;
	int			errs		= 0;
	int			i;
	int			tag;

	dev	= &global.device[index];

	for (i = 0; i < dev->value_qty; i++)
	{
		tag	= (int) ((dev->fifo[i] >> 24) & 0xF);

		if (tag != chan)
		{
			errs	= 1;
			break;
		}

		chan	= (chan + 1) % dev->chan_qty;
	}

	if (errs)
		sprintf(dst, "FAIL <---  (@#%d)", i);
	else
		strcpy(dst, "PASS");

	return(errs);
}



//*****************************************************************************
// Verify thet the number of values per channel is sufficient.
int analyze_channel_values(const args_t* args, int index, char* dst, int channel)
{
	channel_t*	chan;
	device_t*	dev;
	int			errs;
	const char*	factor	= "";
	int			value_qty;

	dev		= &global.device[index];
	chan	= &dev->channel[channel];

	if (channel >= dev->chan_qty)
	{
		errs	= 0;
		strcpy(dst, "N/A");
		dev->chan_vals	= 0;
	}
	else
	{
		value_qty	= chan->value_qty;

		if (value_qty < CHAN_VALS_MIN)
		{
			errs	= 1;
			strcpy(dst, "FAIL <---");
		}
		else
		{
			errs	= 0;
			strcpy(dst, "PASS");
		}

		if ((value_qty % 1024) == 0)
		{
			value_qty	/= 1024;
			factor		= "K";
		}

		strcat(dst, " (");
		gsc_label_long_comma_buf(value_qty, dst + strlen(dst));
		strcat(dst, factor);
		strcat(dst, ")");
	}

	return(errs);
}



//*****************************************************************************
// Count and validate the number of sine wave cycles captured in the input data.
int analyze_count_cycles(const args_t* args, int index, char* dst, int channel)
{
	channel_t*	chan;
	int			cycles		= 0;
	device_t*	dev;
	int			found;
	int			high;
	int			low;
	int			next		= 0;
	int			ref_cycles	= global.device[0].channel[0].cycles;
	int			zero;

	dev		= &global.device[index];
	chan	= &dev->channel[channel];
	chan->cycles	= 0;

	if (channel >= dev->chan_qty)
	{
		strcpy(dst, "N/A");
	}
	else
	{
		dev->index_max	= dev->value_qty - 1;

		for (;;)
		{
			found	= _find_lzh_points(index, channel, &next, &low, &zero, &high);

			if (found == 0)
				break;

			if (cycles < MAX_CYCLE_QTY)
			{
				chan->lzh_list[cycles].low	= low;
				chan->lzh_list[cycles].zero	= zero;
				chan->lzh_list[cycles].high	= high;
				chan->cycles++;
			}

			cycles++;
		}

		if (cycles < CHAN_CYCLES_MIN)
		{
			strcpy(dst, "IGNORE");
			chan->cycles	= 0;
		}
		else if (	((index > 0) || (channel > 0)) &&
					(global.device[0].channel[0].cycles == 0))
		{
			strcpy(dst, "UNUSABLE");
			chan->cycles	= 0;
		}
		else if (	((index > 0) || (channel > 0)) &&
					(	(cycles < (0.95 * ref_cycles)) ||
						(cycles > (1.05 * ref_cycles))))
		{
			strcpy(dst, "INVALID");
			chan->cycles	= 0;
		}
		else
		{
			strcpy(dst, "PASS");
			chan->cycles	= cycles;
		}

		strcat(dst, " (");
		gsc_label_long_comma_buf(cycles, dst + strlen(dst));
		strcat(dst, ")");
	}

	return(0);
}



//*****************************************************************************
// Verify that the width of the sine wave cycles (in samples) is sufficient.
int analyze_cycle_width(const args_t* args, int index, char* dst, int channel)
{
	long double	ave;
	channel_t*	chan;
	device_t*	dev;
	long double	width;

	dev		= &global.device[index];
	chan	= &dev->channel[channel];

	if (chan->cycles <= 1)
	{
		strcpy(dst, "N/A");
	}
	else
	{
		width	= chan->lzh_list[chan->cycles - 1].zero
				- chan->lzh_list[0].zero;
		ave		= (long double) width / (chan->cycles - 1);

		if (ave < CYCLE_WIDTH_MIN)
		{
			sprintf(dst, "TOO LOW (%.1f)", (float) ave);
			chan->cycles	= 0;
			chan->width		= 0;
		}
		else
		{
			sprintf(dst, "PASS (%.1Lf)", ave);
			chan->width		= ave;
		}
	}

	return(0);
}



//*****************************************************************************
int analyze_sample_rate(const args_t* args, int index, char* dst)
{
	int		errs	= 0;
	s32		fref;
	s32		mca;
	s32		mode;
	s32		ndiv;
	s32		nvco;
	s32		nref;
	double	rate;

	if (index == 0)
	{
		errs	+= dsi16wrc_query(args->fd[0], -1, 0, DSI16WRC_QUERY_FREF_DEFAULT, &fref);
		errs	+= dsi16wrc_fsamp_ai_compute(args->fd[0], -1, 0, fref, args->fsamp, &mode, &nvco, &nref, &ndiv, &mca, &rate);

		if (errs)
		{
			strcpy(dst, "FAIL <---");
			global.fsamp	= 0;
		}
		else
		{
			if (rate <= 0)
			{
				strcpy(dst, "FAIL <---");
				global.fsamp	= 0;
			}
			else
			{
				global.fsamp	= rate;
				strcpy(dst, "PASS");
			}

			strcat(dst, " (");
			gsc_label_float_comma_buf(rate, 0, 0, dst + strlen(dst));
			strcat(dst, ")");
		}
	}
	else
	{
		strcpy(dst, "N/A");
	}

	return(errs);
}



//*****************************************************************************
int analyze_separate_data(const args_t* args, int index, char* dst)
{
	channel_t*	chan;
	device_t*	dev;
	int			i;
	int			tag;
	u32			value;

	dev	= &global.device[index];

	for (i = 0; i < dev->chan_qty; i++)
		dev->channel[i].value_qty	= 0;

	for (i = 0; i < dev->value_qty; i++)
	{
		value	= dev->fifo[i];
		tag		= (value >> 24) & 0xF;
		chan	= &dev->channel[tag];
		chan->data[chan->value_qty]	= value;
		chan->value_qty++;
	}

	strcpy(dst, "Done");
	return(0);
}



//*****************************************************************************
int analyze_voltage_range(const args_t* args, int index, char* dst, int channel)
{
	float		ave;
	channel_t*	chan;
	int			delta;
	device_t*	dev;
	int			high;
	int			i;
	int			low;
	float		range;
	long		sum		= 0;

	dev		= &global.device[index];
	chan	= &dev->channel[channel];

	if (chan->cycles <= 0)
	{
		strcpy(dst, "N/A");
	}
	else
	{
		for (i = 0; i < chan->cycles; i++)
		{
			low		= chan->lzh_list[i].low;
			high	= chan->lzh_list[i].high;

			low		= chan->data[low] & 0xFFFF;
			high	= chan->data[high] & 0xFFFF;

			delta	= high - low;
			sum		+= delta;
		}

		ave		= (float) sum / chan->cycles;
		range	= ave / 0x10000 * 20;	// 16-bit resolution, +-10V range

		if (range < 2.0)	// +- 1V
		{
			sprintf(dst, "TOO LOW (%.2fV)", range);
			chan->cycles	= 0;
		}
		else
		{
			sprintf(dst, "PASS (%.2fV)", range);
		}
	}

	return(0);
}



//*****************************************************************************
static long double _compute_x_intercept_lsa(const args_t* args, device_t* dev, channel_t* chan, int x1, int x2)
{
	long double		b;					// y intercept
	long double		dt;					// Delta Time for each X step.
	long double		m;					// slope of line
	int				n;					// number of values summed
	long double		x;					// X intercept
	long double		xf;					// X floating point value
	int				xi;					// X integer value
	long double		xf_sum;				// sum of xf
	long double		xf2;				// xf * xf
	long double		xf2_sum;			// sum of xf * xf
	long double		xfyf;				// xf * yf
	long double		xfyf_sum;			// sum of xf * yf
	long double		yf;					// Y floating point value
	long double		yf_sum;				// sum of yf
	int				yi;					// Y integer value

	dt	= (long double) 1.0 / global.fsamp;

	// Initialize the Lease Squares house keeping variables.
	xf_sum		= 0;
	yf_sum		= 0;
	xf2_sum		= 0;
	xfyf_sum	= 0;

	for (xi = x1; xi <= x2; xi++)
	{
		// Extract integer values
		yi	= (int) (chan->data[xi] & 0xFFFF);

		// Convert to voltage levels
		yf	= ((long double) yi / 65536) * 20 - 10.0;

		// Time
		xf	= dt * xi;

		// Compute Least Squares values.
		xf2			= xf * xf;
		xfyf		= xf * yf;
		xf_sum		+= xf;
		yf_sum		+= yf;
		xf2_sum		+= xf2;
		xfyf_sum	+= xfyf;

		if (args->details)
		{
			printf("%3d. ", __LINE__);
			printf("xi %3d, ", (int) xi);
			printf("(xi %3d, yi %5d), ", xi, yi);
			printf("(xf %15.12Lf, yf %12.9Lf), ", xf, yf);
			printf("xf2 %18.15Lf, ", xf2);
			printf("xfyf %15.12Lf, ", xfyf);
			printf("xf_sum %14.11Lf, ", xf_sum);
			printf("yf_sum %12.9Lf, ", yf_sum);
			printf("xf2_sum %17.14Lf, ", xf2_sum);
			printf("xfyf_sum %15.12Lf", xfyf_sum);
			printf("\n");
		}
	}

	// Finish the Least Squared calculations for the line.
	n	= x2 - x1 + 1;
	m	= (xfyf_sum - xf_sum * yf_sum / n)
		/ (xf2_sum - xf_sum * xf_sum / n);
	b	= (yf_sum / n) - m * (xf_sum / n);
	x	= -b / m;

	if (args->details)
	{
		printf("%3d. ", __LINE__);
		printf("x1 %2d, ", (int) x1);
		printf("x2 %2d, ", (int) x2);
		printf("n %2d, ", (int) n);
		printf("m %12.9Lf, ", m);
		printf("b %12.9Lf, ", b);
		printf("x %12.9Lf", x);
		printf("\n");
	}

	return(x);
}



//*****************************************************************************
// Use Least Squares Analysis to locate the x intercept of the zero corossing,
// rising portion of the first full sine wave in the captured data.
int analyze_lsa(const args_t* args, int index, char* dst, int channel)
{
	channel_t*		chan;
	int				cycle;
	long double		delay;
	device_t*		dev;
	long double		dt;					// Delta Time for each X step.
	int				x1;
	int				x2;
	long double		xf;					// X intercept
	long double		xf_ave;				// average adjusted X intercept
	long double		xf_sum		= 0;	// sum of adjusted X intercepts

	dev		= &global.device[index];
	chan	= &dev->channel[channel];

	for (;;)	// A convenience loop.
	{
		if (chan->cycles <= 0)
		{
			strcpy(dst, "N/A");
			break;
		}

		if (args->details)
			printf("\n\n==== Device %d, Channel %d\n", index, channel);

		dt	= (long double) 1.0 / global.fsamp;

		for (cycle = 0; cycle < chan->cycles; cycle++)
		{
			x1		= (chan->lzh_list[cycle].zero + chan->lzh_list[cycle].low) / 2;
			x2		= (chan->lzh_list[cycle].high + chan->lzh_list[cycle].zero) / 2;
			xf		= _compute_x_intercept_lsa(args, dev, chan, x1, x2);
			xf_sum	+= xf - dt * cycle;

			if (args->details)
			{
				printf("%3d. ", __LINE__);
				printf("cycle %4d, ", (int) cycle);
				printf("low %5d, ", (int) chan->lzh_list[cycle].low);
				printf("zero %5d, ", (int) chan->lzh_list[cycle].zero);
				printf("high %5d, ", (int) chan->lzh_list[cycle].high);
				printf("x1 %5d, ", (int) x1);
				printf("x2 %5d, ", (int) x2);
				printf("xf %12.9Lf, ", xf);
				printf("xf_sum %12.9Lf, ", xf_sum);
				printf("\n");
			}
		}

		xf_ave	= xf_sum / chan->cycles;

		if (args->raw_delay)
		{
			sprintf(dst, "%.9Lf S", xf_ave);
		}
		else
		{
			if ((index == 0) && (channel == 0))
			{
				// Record the reference value.
				global.delta_x_reference	= xf_ave;
			}

			// Report the results.
			delay	= xf_ave - global.delta_x_reference;
			sprintf(dst, "%.3Lf ns", delay * 1000000000);
		}

		break;
	}

	return(0);
}



//*****************************************************************************
int analyze_data(const args_t* args)
{
	int	chan;
	int	errs	= 0;

	printf("-- Analyzing:\n");

	errs	+= service_sample_rate(args);
	errs	+= service_channel_tags(args);
	errs	+= service_separate_data(args);

	for (chan = 0; chan < global.channels_max; chan++)
		errs	+= service_channel_values(args, chan);

	for (chan = 0; chan < global.channels_max; chan++)
		errs	+= service_count_cycles(args, chan);

	for (chan = 0; chan < global.channels_max; chan++)
		errs	+= service_cycle_width(args, chan);

	for (chan = 0; chan < global.channels_max; chan++)
		errs	+= service_voltage_range(args, chan);

	for (chan = 0; chan < global.channels_max; chan++)
		errs	+= service_lsa(args, chan);

	return(errs);
}


