// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/mbsync/main.c $
// $Rev: 52507 $
// $Date: 2023-03-01 12:39:11 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



// variables ******************************************************************

		global_t	global;



//*****************************************************************************
static int _parse_args(int argc, char** argv, args_t* args)
{
	const gsc_arg_item_t	list[]	=
	{
		//	type				var						arg				values	desc
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-c",			{ -1 },	"Continue testing until an error occurs."							},
		{	GSC_ARG_S32_FLAG,	&args->chan_tag,		"-ct",			{ 0 },	"Exclude the Channel Tag from the saved data."						},
		{	GSC_ARG_S32_FLAG,	&args->continuous,		"-C",			{ +1 },	"Continue testing even if errors occur."							},
		{	GSC_ARG_S32_FLAG,	&args->decimal,			"-d",			{ 1 },	"Save the data in decimal format. (The default is hex.)"			},
		{	GSC_ARG_S32_FLAG,	&args->star,			"-dc",			{ 0 },	"The clock and sync signals are in a DAISY CHAIN configuration."	},
		{	GSC_ARG_S32_FLAG,	&args->ignore,			"-i",			{ 1 },	"Ignore test failures when possible."								},
		{	GSC_ARG_S32_FLAG,	&args->ttl,				"-lvds",		{ 0 },	"Use LVDS transceivers (default)."									},
		{	GSC_ARG_S32_MIN,	&args->minute_limit,	"-m",			{ 1 },	"Continue testing for up to # minutes."								},
		{	GSC_ARG_S32_MIN,	&args->test_limit,		"-n",			{ 1 },	"Continue testing for up to # iterations."							},
		{	GSC_ARG_S32_FLAG,	&args->raw_delay,		"-rd",			{ 1 },	"Show the raw time delay. The default is relative."					},
		{	GSC_ARG_S32_FLAG,	&args->star,			"-s",			{ 1 },	"The clock and sync signals are in a STAR configuration."			},
		{	GSC_ARG_S32_FLAG,	&args->ttl,				"-ttl",			{ 1 },	"Use TTL transceivers."												},
		{	GSC_ARG_S32_FLAG,	&args->sync_none,		"-u",			{ 1 },	"Do not SYNC the Target(s) to the Initiator."						},
		{	GSC_ARG_S32_FLAG,	&args->verbose,			"-v",			{ 1 },	"Generate verbose output."											},
		{	GSC_ARG_S32_FLAG,	&args->details,			"-vcd",			{ 1 },	"Generate verbose output for computational details. Very verbose!"	},
		{	GSC_ARG_DEV_INDEX,	&args->index[0],		"INITIATOR",	{ 9 },	"The zero based index of the INITIATOR device."						},
		{	GSC_ARG_DEV_INDEX,	&args->index[1],		"TARGET_1",		{ 9 },	"The zero based index of the first TARGET device."					},
		{	GSC_ARG_DEV_INDEX,	&args->index[2],		"TARGET_2",		{ 9 },	"The zero based index of the second TARGET device."					},
		{	GSC_ARG_DEV_INDEX,	&args->index[3],		"TARGET_3",		{ 9 },	"The zero based index of the third TARGET device."					},
		{	GSC_ARG_DEV_INDEX,	&args->index[4],		"TARGET_4",		{ 9 },	"The zero based index of the forth TARGET device."					},
		{	GSC_ARG_DEV_INDEX,	&args->index[5],		"TARGET_5",		{ 9 },	"The zero based index of the fifth TARGET device."					},
		{	GSC_ARG_DEV_INDEX,	&args->index[6],		"TARGET_6",		{ 9 },	"The zero based index of the sixth TARGET device."					},
		{	GSC_ARG_DEV_INDEX,	&args->index[7],		"TARGET_7",		{ 9 },	"The zero based index of the seventh TARGET device."				},
		{	GSC_ARG_DEV_INDEX,	&args->index[8],		"TARGET_8",		{ 9 },	"The zero based index of the eighth TARGET device."					}
	};

	const gsc_arg_set_t	set	=
	{
		/* name	*/	"mbsync",
		/* desc	*/	"Multi-Board Synchronization demonstrator.",
		/* qty	*/	SIZEOF_ARRAY(list),
		/* list	*/	list
	};

	int	errs;
	int	i;

	memset(&global, 0, sizeof(global));
	memset(args, 0, sizeof(args_t));
	args->star	= -1;

	for (i = 0; i < MAX_BOARD_QTY; i++)
		args->index[i]	= -1;

	args->chan_tag		= 1;
	args->format		= DSI16WRC_DATA_FORMAT_OFF_BIN;
	args->fsamp			= SAMPLE_RATE;
	args->index[0]		= 0;
	args->index[1]		= 1;
	args->io_overflow	= DSI16WRC_IO_OVERFLOW_IGNORE;
	args->qty			= 0;
	args->range			= -1;
	args->width			= DSI16WRC_DATA_WIDTH_16;

	errs	= gsc_args_parse(argc, argv, &set);

	for (i = 0; i < MAX_BOARD_QTY; i++)
		args->qty	+= (args->index[i] >= 0) ? 1 : 0;

	if (errs)
	{
	}
	else if (args->star < 0)
	{
		errs	= 1;
		printf("FAIL <---  (Must specify STAR or DAISY-CHAIN configuration.\n");
	}

	return(errs);
}



//*****************************************************************************
static void _show_access_index(const args_t* args)
{
	int	i;

	gsc_label("Accessing Devices");

	printf("Initiator: %d", (int) args->index[0]);
	printf(", Targets: %d", (int) args->index[1]);

	for (i = 2; i < (args->qty - 1); i++)
		printf(", %d", (int) args->index[i]);

	if (args->qty >= 3)
		printf(" and %d", (int) args->index[i]);

	printf("\n");
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were encounterred.
*		EXIT_FAILURE	One or more problems were encounterred.
*
******************************************************************************/

int main(int argc, char** argv)
{
	args_t	args;
	int		errs		= 0;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty			= 0;
	int		ret;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;

	for (;;)
	{
		gsc_label_init(29);
		test	= time(NULL);
		errs	= _parse_args(argc, argv, &args);

		if (errs)
			break;

		// Introduction

		t_limit	= exec + (args.minute_limit * 60);
		os_id_host();
		_show_time_stamp();
		_show_access_index(&args);
		errs	+= dsi16wrc_init_util(1);
		errs	+= os_id_driver(dsi16wrc_open, dsi16wrc_read, dsi16wrc_close);
		errs	+= dsi16wrc_count_boards(1, &qty);

		if (qty < 2)
			errs++;

		if (errs)
			args.continuous	= 0;

		// Open access to devices.

		if (errs == 0)
			errs	+= service_open(&args, args.fd);

		if (errs == 0)
		{
			// Device identification

			errs	+= service_id_device(&args);

			// Perform testing

			errs	+= perform_tests(&args);
		}

		// Close access

		errs	+= service_close(&args);

		// End of processing.

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= (long) (now - test);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= (long) (now - exec);
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if (args.continuous == 0)
			break;

		if ((args.continuous < 0) && (errs))
			break;

		if ((args.minute_limit > 0) && (now >= t_limit))
			break;

		if ((args.test_limit > 0) && (tests >= args.test_limit))
			break;
	}

	ret	= ((qty <= 0) || (errs) || (failures)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}


