// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/mbsync/main.h $
// $Rev: 52507 $
// $Date: 2023-03-01 12:39:11 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "24dsi16wrc_main.h"



// macros *********************************************************************

#define	MAX_BOARD_QTY			9	// maximum number of devices to be sync'd
#define	MAX_CYCLE_QTY			_16K

#define	SAMPLE_RATE				(13316L)
#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))

#define	_8K						(8 * 1024)
#define	_16K					(16 * 1024)
#define	_256K					(256 * 1024)



// data types *****************************************************************

typedef struct
{
	int			cycles;		// Number of cycles detected in data.
	int			value_qty;	// Number of values in data array.
	long double	width;		// Average width of each cycle in points.

	u32			data[_256K];

	struct
	{
		int	low;		// Index of sine wave low peak.
		int	zero;		// Index of first zero negative value after "low".
		int	high;		// Index of sine wave high peak following "zero".
	} lzh_list[MAX_CYCLE_QTY];
} channel_t;

typedef struct
{
	int			errs;			// from the read process
	int			chan_qty;
	int			chan_vals;		// Number of values per channel.
	int			index_max;		// Maximum index of value to examine.
	int			value_qty;		// Number of values read into the below FIFO buffer.

	u32			fifo[_256K];

	channel_t	channel[16];
} device_t;

typedef struct
{
	int			channels_max;
	long double	delta_x_reference;
	double		fsamp;
	device_t	device[MAX_BOARD_QTY];
} global_t;

typedef struct
{
	// Application Settings

	s32	chan_tag;				// Include the channel tag in the saced data?
	s32	continuous;				// > 0 = ignore errors, < 0 = stop on errors
	s32	decimal;				// Save the data values in decimal format? Hex otherwise.
	s32	details;				// Show computational details?
	s32	fsamp;
	s32	ignore;					// Ignore processing errors when possible.
	s32	index[MAX_BOARD_QTY];	// Index of devices to access.
	s32	io_overflow;
	s32	minute_limit;
	s32	qty;					// Number of devices being accessed.
	s32	raw_delay;				// Show the raw time delay value, rather than relative.
	s32	star;					// Star configuration? If no, then Daisy Chain.
	s32	sync_none;
	s32	test_limit;
	s32	verbose;

	int	fd[MAX_BOARD_QTY];

	// Device Settings

	s32	format;
	s32	range;
	s32	ttl;					// Use TTL? If not, then LVDS.
	s32	width;
} args_t;



// variables ******************************************************************

extern	global_t		global;



// prototypes *****************************************************************

int		analyze_channel_tags	(const args_t* args, int index, char* dst);
int		analyze_channel_values	(const args_t* args, int index, char* dst, int channel);
int		analyze_count_cycles	(const args_t* args, int index, char* dst, int channel);
int		analyze_cycle_width		(const args_t* args, int index, char* dst, int channel);
int		analyze_data			(const args_t* args);
int		analyze_lsa				(const args_t* args, int index, char* dst, int channel);
int		analyze_sample_rate		(const args_t* args, int index, char* dst);
int		analyze_separate_data	(const args_t* args, int index, char* dst);
int		analyze_voltage_range	(const args_t* args, int index, char* dst, int channel);

int		multi_board_sync		(const args_t* args);

int		perform_tests			(const args_t* args);
int		process_data			(const args_t* args);
int		process_save_data		(const args_t* args, int index);

int		service_ai_buf_clear	(const args_t* args);
int		service_ai_buf_enable	(const args_t* args, int initiator, int target);
int		service_ai_buf_fill_lvl	(const args_t* args);
int		service_ai_buf_has_data	(const args_t* args);
int		service_ai_buf_is_empty	(const args_t* args);
int		service_auto_calibrate	(const args_t* args);
int		service_ch_grp_src		(const args_t* args, int initiator, int target);
int		service_channel_qty		(const args_t* args);
int		service_channel_tags	(const args_t* args);
int		service_channel_values	(const args_t* args, int channel);
int		service_channels_ready	(const args_t* args);
int		service_close			(const args_t* args);
int		service_config_ai		(const args_t* args);
int		service_control_mode	(const args_t* args, int initiator, int target);
int		service_count_cycles	(const args_t* args, int channel);
int		service_cycle_width		(const args_t* args, int channel);
int		service_data_format		(const args_t* args);
int		service_data_status		(const args_t* args);
int		service_data_width		(const args_t* args);
int		service_delay_ms		(const args_t* args, int initiator, int target);
int		service_ext_clk_src		(const args_t* args, int initiator, int target);
int		service_fsamp_ai		(const args_t* args);
int		service_id_device		(const args_t* args);
int		service_lsa				(const args_t* args, int channel);
int		service_open			(const args_t* args, int* fd);
int		service_range			(const args_t* args);
int		service_read_data		(const args_t* args);
int		service_rx_io_overflow	(const args_t* args);
int		service_sample_rate		(const args_t* args);
int		service_save_data		(const args_t* args);
int		service_separate_data	(const args_t* args);
int		service_sw_sync			(const args_t* args);
int		service_sw_sync_mode	(const args_t* args, int initiator, int target);
int		service_voltage_range	(const args_t* args, int channel);
int		service_xcvr_type		(const args_t* args);



#endif
