// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/mbsync/process.c $
// $Rev: 52507 $
// $Date: 2023-03-01 12:39:11 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _save_header_entry(const args_t* args, FILE* file, int device, int line, int chan)
{
	char	buf[64]	= "";
	int		errs;
	int		i;
	int		len;

	switch (line)
	{
		default:
		case 1:

			if (device == 0)
				strcpy(buf, "# Initiator\n");
			else
				sprintf(buf, "# Target %d\n", device);

			break;

		case 2:

			if (chan == -1)
				strcpy(buf, "# Timestamp   ");
			else if (chan == (global.device[device].chan_qty -1))
				sprintf(buf, "  Chan %d\n", chan);
			else if (chan < 10)
				sprintf(buf, "  Chan %d  ", chan);
			else
				sprintf(buf, "  Chan %d ", chan);

			break;

		case 3:

			if (chan == -1)
				strcpy(buf, "# ============");
			else if (chan == (global.device[device].chan_qty -1))
				strcpy(buf, "  ========\n");
			else
				strcpy(buf, "  ========");

			break;
	}

	len		= (int) strlen(buf);
	i		= fprintf(file, "%s", buf);
	errs	= (i == len) ? 0 : 1;
	return(errs);
}



//*****************************************************************************
static int _save_headers(const args_t* args, int device, FILE* file)
{
	int	errs	= 0;
	int	i;

	// File Content
	errs	+= _save_header_entry(args, file, device, 1, 0);

	// Titles

	for (i = -1; i < global.device[device].chan_qty; i++)
		errs	+= _save_header_entry(args, file, device, 2, i);

	// Underscores

	for (i = -1; i < global.device[device].chan_qty; i++)
		errs	+= _save_header_entry(args, file, device, 3, i);

	errs	= errs ? 1 : 0;
	return(errs);
}



//*****************************************************************************
static int _save_data_line(const args_t* args, FILE* file, int device, long double t0, int index)
{
	char	buf[64];
	int		errs	= 0;
	int		i;
	int		j;
	int		len;

	sprintf(buf, "%14.9Lf", t0);
	len	= (int) strlen(buf);
	j		= fprintf(file, "%s", buf);
	errs	+= (j == len) ? 0 : 1;

	for (i = 0; i < global.device[device].chan_qty; i++)
	{
		if ((i + index) >= global.device[device].value_qty)
			break;

		if (args->chan_tag)
		{
			if (args->decimal)
				sprintf(buf, "% 10d", (int) (global.device[device].fifo[i + index] & 0xFFFFFFFF));
			else
				sprintf(buf, "  %08lX", (long) (global.device[device].fifo[i + index] & 0xFFFFFFFF));
		}
		else
		{
			if (args->decimal)
				sprintf(buf, "% 10d", (int) (global.device[device].fifo[i + index] & 0xFFFFFF));
			else
				sprintf(buf, "  %06lX  ", (long) (global.device[device].fifo[i + index] & 0xFFFFFF));
		}

		len	= (int) strlen(buf);
		j		= fprintf(file, "%s", buf);
		errs	+= (j == len) ? 0 : 1;
	}

	j		= fprintf(file, "\n");
	errs	+= (j == 1) ? 0 : 1;
	errs	= errs ? 1 : 0;
	return(errs);
}



//*****************************************************************************
int process_data(const args_t* args)
{
	int	errs	= 0;

	printf("-- Post Processing:\n");

	errs	+= service_ai_buf_fill_lvl	(args);
	errs	+= service_read_data		(args);
	errs	+= service_channel_qty		(args);
	errs	+= service_data_status		(args);
	errs	+= service_save_data		(args);

	return(errs);
}



//*****************************************************************************
int process_save_data(const args_t* args, int index)
{
	long double	delta			= (long double) 1.0 / args->fsamp;
	device_t*	dev;
	int			errs;
	FILE*		file;
	int			i;
	char		name[64];
	long double	t0				= 0;

	dev	= &global.device[index];

	if (index == 0)
		strcpy(name, "mbsync_initiator.txt");
	else
		sprintf(name, "mbsync_target_%d.txt", index);

	file	= fopen(name, "w+b");

	if (file == NULL)
	{
		errs	= 1;
	}
	else
	{
		errs	= _save_headers(args, index, file);

		for (i = 0; i < dev->value_qty; i += dev->chan_qty)
		{
			errs	= _save_data_line(args, file, index, t0, i);
			t0		+= delta;
		}

		fclose(file);
	}

	return(errs);
}



