// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/mbsync/service.c $
// $Rev: 52507 $
// $Date: 2023-03-01 12:39:11 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	TITLE_LENGTH			15
#define	FIELD_LENGTH			18



// prototypes *****************************************************************

static	int	_service_vendor_id			(const args_t* args);
static	int	_service_subsystem_vendor_id(const args_t* args);
static	int	_service_subsystem_device_id(const args_t* args);



//*****************************************************************************
static void _show_field(int width, const char* title)
{
	static char	buf[1024];

	int	i;
	int	len;

	if (width >= (sizeof(buf) - 2))
		width	= sizeof(buf) - 2;

	if (title == NULL)
		title	= "";

	strncpy(buf, title, sizeof(buf));
	buf[sizeof(buf) - 1]	= 0;
	buf[sizeof(buf) - 2]	= 0;
	buf[width]				= 0;

	if (strcmp(title, "=") == 0)
	{
		// Make the equal equal a bunch of '=' chars.

		for (i = 0; i < width; i++)
			buf[i]	= '=';

		buf[i]	= 0;
	}

	len	= (int) strlen(buf);

	for (; len < width; len++)
		strcat(buf, " ");

	printf("%s  ", buf);
	fflush(stdout);
}



//*****************************************************************************
static void _service_data(const char* data)
{
	_show_field(FIELD_LENGTH, data);
}



//*****************************************************************************
static void _service_title(const char* title)
{
	_show_field(TITLE_LENGTH, title);
}



//*****************************************************************************
static void _service_titles(const args_t* args)
{
	char	buffer[128];
	int		i;

	printf("\n");

	//=====================================================
	_service_title("Operation");

	sprintf(buffer, "Initiator (#%d)", args->index[0]);
	_service_data(buffer);

	for (i = 1; i < args->qty; i++)
	{
		sprintf(buffer, "Target %d (#%d)", i, args->index[i]);
		_service_data(buffer);
	}

	printf("Status\n");

	//=====================================================
	_service_title("=");

	for (i = 0; i < args->qty; i++)
		_service_data("=");

	_service_data("=");
	printf("\n");
}



//*****************************************************************************
static int _service_entry(
	const args_t*	args,
	const char*		title,
	int				initiator,
	int				target,
	int				(*func)(const args_t*	args,
							int				index,
							char*			dst,
							int				initiator,
							int				target),
	int				verbose_ok)
{
	char	buffer[128];
	int		err;
	int		errs			= 0;
	int		i;

	if ((args->verbose) && (verbose_ok))
	{
		gsc_label(title);
		printf("\n");
		gsc_label_level_inc();
	}
	else
	{
		_service_title(title);
	}

	for (i = 0; i < args->qty; i++)
	{
		if ((args->ignore == 0) && (global.device[i].errs))
		{
			if ((args->verbose == 0) || (verbose_ok == 0))
				_service_data("Skipped");
		}
		else
		{
			err	= (func)(args, i, buffer, initiator, target);
			errs	+= err ? 1 : 0;

			if ((args->verbose == 0) || (verbose_ok == 0))
				_service_data(buffer);
		}
	}

	if ((args->verbose) && (verbose_ok))
	{
		gsc_label_level_dec();
	}
	else
	{
		printf("%s\n", errs ? "FAIL <---" : "PASS");
	}

	return(errs);
}



//*****************************************************************************
static int _callback_ai_buf_clear(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;

	errs	= dsi16wrc_ai_buf_clear(args->fd[index], -1, 0);
	sprintf(dst, "%s", errs ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
int service_ai_buf_clear(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Clear Buffer", 0, 0, _callback_ai_buf_clear, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_ai_buf_enable(const args_t* args, int index, char* dst, int initiator, int target)
{
	int	errs;
	s32	get;
	s32	set		= (index == 0) ? initiator : target;

	errs	= dsi16wrc_ai_buf_enable(args->fd[index], -1, 0, set, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_AI_BUF_ENABLE_NO)
	{
		strcpy(dst, "Disabled");
	}
	else if (get == DSI16WRC_AI_BUF_ENABLE_YES)
	{
		strcpy(dst, "Enabled");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_ai_buf_enable(const args_t* args, int initiator, int target)
{
	int	errs;

	errs	= _service_entry(args, "Buffer Enable", initiator, target, _callback_ai_buf_enable, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_ai_buf_fill_lvl(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	int	i;
	s32	last	= -1;
	s32	get;

	for (i = 0;; i++)
	{
		errs	= dsi16wrc_ai_buf_fill_lvl(args->fd[index], -1, 0, &get);

		if (errs)
		{
			strcpy(dst, "FAIL <---");
			break;
		}

		if (get == _256K)
		{
			strcpy(dst, "PASS (Full)");
			break;
		}

		if (get == last)
		{
			errs	= 1;
			strcpy(dst, "NOT FILLING <---");
			break;
		}

		if (i == 1000)
		{
			errs	= 1;
			strcpy(dst, "TIMEOUT <---");
			break;
		}

		last	= get;
		os_sleep_ms(10);
	}

	return(errs);
}



//*****************************************************************************
int service_ai_buf_fill_lvl(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "FIFO Full", 0, 0, _callback_ai_buf_fill_lvl, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_ai_buf_has_data(const args_t* args, int index, char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	errs	= dsi16wrc_ai_buf_fill_lvl(args->fd[index], -1, 0, &get);

	if (errs)
	{
		errs	= 1;
		strcpy(dst, "FAIL <---");
	}
	else if (get > 0)
	{
		strcpy(dst, "Has Data");
	}
	else
	{
		errs	= 1;
		strcpy(dst, "EMPTY <---");
	}

	return(errs);
}



//*****************************************************************************
int service_ai_buf_has_data(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Buffer Status", 0, 0, _callback_ai_buf_has_data, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_ai_buf_is_empty(const args_t* args, int index, char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	errs	= dsi16wrc_ai_buf_fill_lvl(args->fd[index], -1, 0, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get > 0)
	{
		errs	= 1;
		strcpy(dst, "Has Data <---");
	}
	else
	{
		strcpy(dst, "Empty");
	}

	return(errs);
}



//*****************************************************************************
int service_ai_buf_is_empty(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Buffer Status", 0, 0, _callback_ai_buf_is_empty, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_auto_calibrate(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;

	errs	= dsi16wrc_auto_calibrate(args->fd[index], -1, 0);
	strcpy(dst, errs ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
int service_auto_calibrate(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Auto-Cal.", 0, 0, _callback_auto_calibrate, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_ch_grp_0_src(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	if (index == 0)
		errs	= dsi16wrc_ch_grp_0_src(args->fd[index], -1, 0, initiator, &get);
	else
		errs	= dsi16wrc_ch_grp_0_src(args->fd[index], -1, 0, target, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_CH_GRP_SRC_RATE_GEN)
	{
		strcpy(dst, "Rate Gen");
	}
	else if (get == DSI16WRC_CH_GRP_SRC_EXTERN)
	{
		strcpy(dst, "External");
	}
	else if (get == DSI16WRC_CH_GRP_SRC_DIR_EXTERN)
	{
		strcpy(dst, "Direct Extern");
	}
	else if (get == DSI16WRC_CH_GRP_SRC_DISABLE)
	{
		strcpy(dst, "Disable");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
static int _callback_ch_grp_1_src(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	if (index == 0)
		errs	= dsi16wrc_ch_grp_1_src(args->fd[index], -1, 0, initiator, &get);
	else
		errs	= dsi16wrc_ch_grp_1_src(args->fd[index], -1, 0, target, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_CH_GRP_SRC_RATE_GEN)
	{
		strcpy(dst, "Rate Gen");
	}
	else if (get == DSI16WRC_CH_GRP_SRC_EXTERN)
	{
		strcpy(dst, "External");
	}
	else if (get == DSI16WRC_CH_GRP_SRC_DIR_EXTERN)
	{
		strcpy(dst, "Direct Extern");
	}
	else if (get == DSI16WRC_CH_GRP_SRC_DISABLE)
	{
		strcpy(dst, "Disable");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_ch_grp_src(const args_t* args, int initiator, int target)
{
	int	errs	= 0;

	errs	+= _service_entry(args, "Ch Grp 0 Src", initiator, target, _callback_ch_grp_0_src, 0);
	errs	+= _service_entry(args, "Ch Grp 1 Src", initiator, target, _callback_ch_grp_1_src, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_channel_qty(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	errs	= dsi16wrc_query(args->fd[index], -1, 0, DSI16WRC_QUERY_CHANNEL_QTY, &get);

	global.device[index].chan_qty	= get;

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get <= 0)
	{
		errs	= 1;
		strcpy(dst, "NONE!!!");
	}
	else
	{
		sprintf(dst, "PASS (%d)", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_channel_qty(const args_t* args)
{
	int	errs;
	int	i;

	errs	= _service_entry(args, "Channel Qty", 0, 0, _callback_channel_qty, 0);

	for (i = 0; i < args->qty; i++)
	{
		if (i == 0)
			global.channels_max	= global.device[i].chan_qty;
		else if (global.channels_max < global.device[i].chan_qty)
			global.channels_max	= global.device[i].chan_qty;
	}

	return(errs);
}



//*****************************************************************************
static int _callback_channel_tags(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;

	errs	= analyze_channel_tags(args, index, dst);
	return(errs);
}



//*****************************************************************************
int service_channel_tags(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Channel Tags", 0, 0, _callback_channel_tags, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_channel_values(const args_t* args, int index, char* dst, int channel, int ignore)
{
	int	errs;

	errs	= analyze_channel_values(args, index, dst, channel);
	return(errs);
}



//*****************************************************************************
int service_channel_values(const args_t* args, int channel)
{
	char	buf[64];
	int		errs;

	if (channel == 0)
		printf("Channel Values:\n");

	sprintf(buf, "  Channel %d", channel);
	errs	= _service_entry(args, buf, channel, 0, _callback_channel_values, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_channels_ready(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	errs	= dsi16wrc_channels_ready(args->fd[index], -1, 0, DSI16WRC_CHANNELS_READY_WAIT, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_CHANNELS_READY_NO)
	{
		errs	= 1;
		strcpy(dst, "NOT READY!!!");
	}
	else if (get == DSI16WRC_CHANNELS_READY_YES)
	{
		strcpy(dst, "Ready");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_channels_ready(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Chans Ready", 0, 0, _callback_channels_ready, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_close(const args_t* args, int index, char* dst, int initiator, int target)
{
	int	errs;
	int	ret;

	ret		= dsi16wrc_close(args->fd[index]);
	errs	= ret ? 1 : 0;
	global.device[index].errs	+= errs;
	strcpy(dst, ret ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
int service_close(const args_t* args)
{
	int	errs;

	_service_titles(args);

	errs	= _service_entry(args, "Close Devices", 0, 0, _callback_close, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_config_ai(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;

	errs	= dsi16wrc_config_ai(args->fd[index], args->index[index], args->verbose, -1, args->fsamp);
	sprintf(dst, "%s", errs ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
int service_config_ai(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Basic Config", 0, 0, _callback_config_ai, 1);
	return(errs);
}



//*****************************************************************************
static int _callback_control_mode(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	if (index == 0)
		errs	= dsi16wrc_control_mode(args->fd[index], -1, 0, initiator, &get);
	else
		errs	= dsi16wrc_control_mode(args->fd[index], -1, 0, target, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_CONTROL_MODE_TARGET)
	{
		strcpy(dst, "Target");
	}
	else if (get == DSI16WRC_CONTROL_MODE_INITIATOR)
	{
		strcpy(dst, "Initiator");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_control_mode(const args_t* args, int initiator, int target)
{
	int	errs;

	errs	= _service_entry(args, "Control Mode", initiator, target, _callback_control_mode, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_count_cycles(const args_t* args, int index, char* dst, int channel, int ignore)
{
	int	errs;

	errs	= analyze_count_cycles(args, index, dst, channel);
	return(errs);
}



//*****************************************************************************
int service_count_cycles(const args_t* args, int channel)
{
	char	buf[64];
	int		errs;

	if (channel == 0)
		printf("Cycles Captured:\n");

	sprintf(buf, "  Channel %d", channel);
	errs	= _service_entry(args, buf, channel, 0, _callback_count_cycles, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_data_format(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	errs	= dsi16wrc_data_format(args->fd[index], -1, 0, args->format, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_DATA_FORMAT_2S_COMP)
	{
		strcpy(dst, "Twos Compliment");
	}
	else if (get == DSI16WRC_DATA_FORMAT_OFF_BIN)
	{
		strcpy(dst, "Offset Binary");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
static int _callback_cycle_width(const args_t* args, int index, char* dst, int channel, int ignore)
{
	int	errs;

	errs	= analyze_cycle_width(args, index, dst, channel);
	return(errs);
}



//*****************************************************************************
int service_cycle_width(const args_t* args, int channel)
{
	char	buf[64];
	int		errs;

	if (channel == 0)
		printf("Cycle Width in Values:\n");

	sprintf(buf, "  Channel %d", channel);
	errs	= _service_entry(args, buf, channel, 0, _callback_cycle_width, 0);
	return(errs);
}



//*****************************************************************************
int service_data_format(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Data Format", 0, 0, _callback_data_format, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_data_status(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;

	if (global.device[index].chan_qty <= 0)
	{
		errs	= 1;
		strcpy(dst, "TO FEW CHANNELS");
	}
	else if (global.device[index].chan_qty > 16)
	{
		errs	= 1;
		strcpy(dst, "TO MANY CHANNELS");
	}
	else if (global.device[index].value_qty < _8K)
	{
		errs	= 1;
		strcpy(dst, "TO LITTLE DATA");
	}
	else
	{
		errs	= 0;
		strcpy(dst, "PASS");
	}

	return(errs);
}



//*****************************************************************************
int service_data_status(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Data Status", 0, 0, _callback_data_status, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_data_width(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	errs	= dsi16wrc_data_width(args->fd[index], -1, 0, args->width, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_DATA_WIDTH_16)
	{
		strcpy(dst, "16-bits");
	}
	else if (get == DSI16WRC_DATA_WIDTH_18)
	{
		strcpy(dst, "18-bits");
	}
	else if (get == DSI16WRC_DATA_WIDTH_20)
	{
		strcpy(dst, "20-bits");
	}
	else if (get == DSI16WRC_DATA_WIDTH_24)
	{
		strcpy(dst, "24-bits");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_data_width(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Data Width", 0, 0, _callback_data_width, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_delay_ms(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	ms		= (index == 0) ? initiator : target;

	if (ms > 0)
	{
		os_sleep_ms(ms);
		sprintf(dst, "PASS (%d ms)", ms);
	}
	else
	{
		strcpy(dst, "N/A");
	}

	return(0);
}



//*****************************************************************************
int service_delay_ms(const args_t* args, int initiator, int target)
{
	char	buf[64];
	int		errs;

	sprintf(buf, "Delay %d ms", initiator);
	errs	= _service_entry(args, buf, initiator, target, _callback_delay_ms, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_device_id(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	u32	reg;

	errs	= dsi16wrc_reg_read(args->fd[index], -1, 0, GSC_PCI_9056_DIDR, &reg);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (reg == 0x9056)
	{
		strcpy(dst, "PCI9056");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %04lX", (long) reg);
	}

	return(errs);
}



//*****************************************************************************
static int _service_device_id(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Device ID", 0, 0, _callback_device_id, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_device_type(const args_t* args, int index,  char* dst, int initiator, int target)
{
	s32	arg;
	int	errs;
	int	ret;

	arg		= DSI16WRC_QUERY_DEVICE_TYPE;
	ret		= dsi16wrc_ioctl(args->fd[index], DSI16WRC_IOCTL_QUERY, &arg);

	if (ret)
	{
		errs	= 1;
		strcpy(dst, "FAIL <---");
	}
	else if (arg == GSC_DEV_TYPE_24DSI16WRC)
	{
		errs	= 0;
		strcpy(dst, "24DSI16WRC");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) arg);
	}

	return(errs);
}



//*****************************************************************************
static int _service_device_type(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Device Type", 0, 0, _callback_device_type, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_ext_clk_src(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	if (index == 0)
		errs	= dsi16wrc_ext_clk_src(args->fd[index], -1, 0, initiator, &get);
	else
		errs	= dsi16wrc_ext_clk_src(args->fd[index], -1, 0, target, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if ((index > 0) && (target < 0))
	{
		strcpy(dst, "N/A");
	}
	else if (get == DSI16WRC_EXT_CLK_SRC_GRP_0)
	{
		strcpy(dst, "Group 0");
	}
	else if (get == DSI16WRC_EXT_CLK_SRC_GEN)
	{
		strcpy(dst, "Rate Gen");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_ext_clk_src(const args_t* args, int initiator, int target)
{
	int	errs;

	errs	= _service_entry(args, "Ext Clk Src", initiator, target, _callback_ext_clk_src, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_fsamp_ai(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	if (index == 0)
	{
		errs	= dsi16wrc_fsamp_ai_report(args->fd[index], -1, 0, 0, &get);

		if (errs)
		{
			strcpy(dst, "FAIL <---");
		}
		else
		{
			gsc_label_long_comma_buf(get, dst);
			strcat(dst, " S/S");
		}
	}
	else
	{
		errs	= 0;
		strcpy(dst, "N/A");
	}

	return(errs);
}



//*****************************************************************************
int service_fsamp_ai(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Sample Rate", 0, 0, _callback_fsamp_ai, 0);
	return(errs);
}



//*****************************************************************************
int service_id_device(const args_t* args)
{
	int	errs	= 0;

	errs	+= _service_device_type			(args);
	errs	+= _service_vendor_id			(args);
	errs	+= _service_device_id			(args);
	errs	+= _service_subsystem_vendor_id	(args);
	errs	+= _service_subsystem_device_id	(args);
	return(errs);
}



//*****************************************************************************
static int _callback_lsa(const args_t* args, int index, char* dst, int channel, int ignore)
{
	int	errs;

	errs	= analyze_lsa(args, index, dst, channel);
	return(errs);
}



//*****************************************************************************
int service_lsa(const args_t* args, int channel)
{
	char	buf[64];
	int		errs;

	if (channel == 0)
		printf("Time Delay: Least Squares Analysis (first full cycle rising intercept)\n");

	sprintf(buf, "  Channel %d", channel);
	errs	= _service_entry(args, buf, channel, 0, _callback_lsa, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_open(const args_t* args, int index, char* dst, int initiator, int target)
{
	int	errs;
	s32	ret;

	ret		= dsi16wrc_open(args->index[index], 0, (void*) &args->fd[index]);
	errs	= ret ? 1 : 0;
	global.device[index].errs	+= errs;
	strcpy(dst, ret ? "FAIL <---" : "PASS");
	return(errs);
}



//*****************************************************************************
int service_open(const args_t* args, int* fd)
{
	int	errs;

	_service_titles(args);

	errs	= _service_entry(args, "Open Devices", 0, 0, _callback_open, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_range(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	errs	= dsi16wrc_range(args->fd[index], -1, 0, args->range, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_RANGE_10MV)
	{
		strcpy(dst, "+-0.01 Volts");
	}
	else if (get == DSI16WRC_RANGE_100MV)
	{
		strcpy(dst, "+-0.1 Volts");
	}
	else if (get == DSI16WRC_RANGE_1V)
	{
		strcpy(dst, "+-1  Volt");
	}
	else if (get == DSI16WRC_RANGE_1_25V)
	{
		strcpy(dst, "+-1.25 Volts");
	}
	else if (get == DSI16WRC_RANGE_2_5V)
	{
		strcpy(dst, "+-2.5 Volts");
	}
	else if (get == DSI16WRC_RANGE_5V)
	{
		strcpy(dst, "+-5 Volts");
	}
	else if (get == DSI16WRC_RANGE_10V)
	{
		strcpy(dst, "+-10 Volts");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_range(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Voltage Range", 0, 0, _callback_range, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_read_data(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	int	get		= (int) sizeof(global.device[index].fifo);
	int	got;

	if (get != (_256K * 4))
		got	= -EFAULT;
	else
		got	= dsi16wrc_read_util(args->fd[index], -1, 0, global.device[index].fifo, get);

	global.device[index].value_qty	= got / 4;

	if (got < 0)
	{
		errs	= 1;
		strcpy(dst, "FAIL <---");
	}
	else if (got == 0)
	{
		errs	= 1;
		strcpy(dst, "NO DATA <---");
	}
	else if (got < get)
	{
		errs	= 1;
		strcpy(dst, "PARTIAL <---");
	}
	else
	{
		errs	= 0;
		strcpy(dst, "PASS (256K)");
	}

	return(errs);
}



//*****************************************************************************
int service_read_data(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Read Data", 0, 0, _callback_read_data, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_rx_io_overflow(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	errs	= dsi16wrc_rx_io_overflow(args->fd[index], -1, 0, args->io_overflow, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_IO_OVERFLOW_IGNORE)
	{
		strcpy(dst, "Ignore");
	}
	else if (get == DSI16WRC_IO_OVERFLOW_CHECK)
	{
		strcpy(dst, "Check");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_rx_io_overflow(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "I/O Overflow", 0, 0, _callback_rx_io_overflow, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_sample_rate(const args_t* args, int index, char* dst, int channel, int ignore)
{
	int	errs;

	errs	= analyze_sample_rate(args, index, dst);
	return(errs);
}



//*****************************************************************************
int service_sample_rate(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Sample Rate", 0, 0, _callback_sample_rate, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_save_data(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;

	errs	= process_save_data(args, index);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else
	{
		strcpy(dst, "PASS");
	}

	return(errs);
}



//*****************************************************************************
int service_save_data(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Save Data", 0, 0, _callback_save_data, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_separate_data(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;

	errs	= analyze_separate_data(args, index, dst);
	return(errs);
}



//*****************************************************************************
int service_separate_data(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Separate Data", 0, 0, _callback_separate_data, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_subsystem_device_id(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	u32	reg;

	errs	= dsi16wrc_reg_read(args->fd[index], -1, 0, GSC_PCI_9056_SID, &reg);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (reg == 0x3466)
	{
		sprintf(dst, "OK (0x%04lX)", (long) reg);
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %04lX", (long) reg);
	}

	return(errs);
}



//*****************************************************************************
static int _service_subsystem_device_id(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Subsystem ID", 0, 0, _callback_subsystem_device_id, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_subsystem_vendor_id(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	u32	reg;

	errs	= dsi16wrc_reg_read(args->fd[index], -1, 0, GSC_PCI_9056_SVID, &reg);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (reg == 0x10B5)
	{
		sprintf(dst, "PLX (0x%04lX)", (long) reg);
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %04lX", (long) reg);
	}

	return(errs);
}



//*****************************************************************************
static int _service_subsystem_vendor_id(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Sub Vendor ID", 0, 0, _callback_subsystem_vendor_id, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_sw_sync(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;

	if (index == 0)
	{
		errs	= dsi16wrc_sw_sync(args->fd[index], -1, 0);
		strcpy(dst, errs ? "FAIL <---" : "PASS");
	}
	else
	{
		errs	= 0;
		strcpy(dst, "N/A");
	}

	return(errs);
}



//*****************************************************************************
int service_sw_sync(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "SW Sync", 0, 0, _callback_sw_sync, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_sw_sync_mode(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;

	if (index == 0)
		errs	= dsi16wrc_sw_sync_mode(args->fd[index], -1, 0, initiator, &get);
	else
		errs	= dsi16wrc_sw_sync_mode(args->fd[index], -1, 0, target, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get == DSI16WRC_SW_SYNC_MODE_SYNC)
	{
		strcpy(dst, "SW Sync");
	}
	else if (get == DSI16WRC_SW_SYNC_MODE_CLR_BUF)
	{
		strcpy(dst, "Clear Buffer");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_sw_sync_mode(const args_t* args, int initiator, int target)
{
	int	errs;

	errs	= _service_entry(args, "SW Sync Mode", initiator, target, _callback_sw_sync_mode, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_vendor_id(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	u32	reg;

	errs	= dsi16wrc_reg_read(args->fd[index], -1, 0, GSC_PCI_9056_VIDR, &reg);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (reg == 0x10B5)
	{
		strcpy(dst, "PLX (0x10B5)");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %04lX", (long) reg);
	}

	return(errs);
}



//*****************************************************************************
static int _service_vendor_id(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Vendor ID", 0, 0, _callback_vendor_id, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_voltage_range(const args_t* args, int index, char* dst, int channel, int ignore)
{
	int	errs;

	errs	= analyze_voltage_range(args, index, dst, channel);
	return(errs);
}



//*****************************************************************************
int service_voltage_range(const args_t* args, int channel)
{
	char	buf[64];
	int		errs;

	if (channel == 0)
		printf("Peak-to-Peak Voltage Range:\n");

	sprintf(buf, "  Channel %d", channel);
	errs	= _service_entry(args, buf, channel, 0, _callback_voltage_range, 0);
	return(errs);
}



//*****************************************************************************
static int _callback_xcvr_type(const args_t* args, int index,  char* dst, int initiator, int target)
{
	int	errs;
	s32	get;
	s32	set	= args->ttl ? DSI16WRC_XCVR_TYPE_TTL : DSI16WRC_XCVR_TYPE_LVDS;

	errs	= dsi16wrc_xcvr_type(args->fd[index], -1, 0, set, &get);

	if (errs)
	{
		strcpy(dst, "FAIL <---");
	}
	else if (get != set)
	{
		errs	= 1;
		strcpy(dst, "FAILED REQUEST");
	}
	else if (get == DSI16WRC_XCVR_TYPE_LVDS)
	{
		strcpy(dst, "LVDS");
	}
	else if (get == DSI16WRC_XCVR_TYPE_TTL)
	{
		strcpy(dst, "TTL");
	}
	else
	{
		errs	= 1;
		sprintf(dst, "UNKNOWN: %d", (int) get);
	}

	return(errs);
}



//*****************************************************************************
int service_xcvr_type(const args_t* args)
{
	int	errs;

	errs	= _service_entry(args, "Transceivers", 0, 0, _callback_xcvr_type, 0);
	return(errs);
}


