// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/rxrate/perform.c $
// $Rev: 52501 $
// $Date: 2023-03-01 12:32:23 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1M	(1024L * 1024L)



// variables ******************************************************************

static u32	_rx[_1M];



//******************************************************************************
static int _rate_test(const args_t* args, long get)
{
	struct timeval	begin;
	int				errs	= 0;
	long			got;
	long long		limit	= (long long) args->rx_mb * 1000000L;
	struct timeval	minimum;
	struct timeval	now;
	long long		sps;
	long long		total	= 0;
	long			us		= 1000000L * args->seconds;

	gsc_label("Reading");

	gettimeofday(&begin, NULL);
	minimum.tv_sec	= begin.tv_sec + args->seconds;
	minimum.tv_usec	= begin.tv_usec;

	for (;;)
	{
		got	= dsi16wrc_read(args->fd, _rx, get);

		if (got < 0)
		{
			errs++;
			printf("FAIL <--- (dsi16wrc_read");
			printf(", total so far %lld", total);
			printf(", requested %ld bytes", (long) get);
			printf(", error %ld)\n", got);
			break;
		}

		gettimeofday(&now, NULL);
		total	+= got;

		if ((total >= limit) && (now.tv_sec > minimum.tv_sec))
		{
			us	= (now.tv_sec - begin.tv_sec) * 1000000L
				+ (now.tv_usec - begin.tv_usec);
			sps	= (total * 10000000L / 4 / us + 5) / 10;

			printf(	"PASS  (");
			gsc_label_long_comma(total / 4);
			printf(" Samples, %ld.%06ld Seconds, ", us / 1000000, us % 1000000);
			gsc_label_long_comma(sps);
			printf(" S/S)\n");
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int		errs	= 0;
	long	get		= sizeof(_rx);

	errs	+= dsi16wrc_config_ai		(args->fd, -1, 1, -1, 105000L);
	errs	+= dsi16wrc_rx_io_mode		(args->fd, -1, 1, args->io_mode, NULL);
	errs	+= dsi16wrc_rx_io_overflow	(args->fd, -1, 1, DSI16WRC_IO_OVERFLOW_IGNORE, NULL);
	errs	+= dsi16wrc_rx_io_underflow	(args->fd, -1, 1, DSI16WRC_IO_UNDERFLOW_IGNORE, NULL);

	gsc_label ("Read Rate");
	printf("(%d second minimum, %ld MB minimum)\n", args->seconds, (long) args->rx_mb);
	gsc_label_level_inc();

	errs	+= dsi16wrc_ai_buf_clear	(args->fd, -1, 1);
	errs	+= dsi16wrc_ai_buf_overflow	(args->fd, -1, 1, DSI16WRC_AI_BUF_OVERFLOW_CLEAR, NULL);
	errs	+= dsi16wrc_ai_buf_underflow(args->fd, -1, 1, DSI16WRC_AI_BUF_UNDERFLOW_CLEAR, NULL);

	errs	+= _rate_test(args, get);

	errs	+= dsi16wrc_ai_buf_overflow	(args->fd, -1, 1, -1, NULL);
	errs	+= dsi16wrc_ai_buf_underflow(args->fd, -1, 1, -1, NULL);

	gsc_label_level_dec();
	return(errs);
}



