// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/sbtest/ai_buf_clear.c $
// $Rev: 52509 $
// $Date: 2023-03-01 12:40:28 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	D20					(1 << 20)



//*****************************************************************************
static int _service_test(const args_t* args)
{
	return(0);
}



//*****************************************************************************
static int _function_test(const args_t* args)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_INITIALIZE,
			/* arg		*/	0,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_AI_BUF_THRESH,
			/* arg		*/	100,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_AI_BUF_ENABLE,
			/* arg		*/	DSI16WRC_AI_BUF_ENABLE_YES,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_SLEEP_MS,
			/* cmd		*/	0,
			/* arg		*/	100,
			/* reg		*/	DSI16WRC_GSC_BCTLR,
			/* mask		*/	0x4000,
			/* value	*/	0x4000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_AI_BUF_ENABLE,
			/* arg		*/	DSI16WRC_AI_BUF_ENABLE_NO,
			/* reg		*/	-1,
			/* mask		*/	0,
			/* value	*/	0
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_AI_BUF_CLEAR,
			/* arg		*/	0,
			/* reg		*/	DSI16WRC_GSC_BCTLR,
			/* mask		*/	0x4000,
			/* value	*/	0x0000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= dsi16wrc_initialize			(args->fd, -1, 0);
	errs	+= service_ioctl_set_reg_list	(args->fd, list);
	errs	+= service_ioctl_set_reg_list	(args->fd, list);
	errs	+= dsi16wrc_initialize			(args->fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _timing_test(const args_t* args)
{
	long long		delta;
	int				errs	= 0;
	int				i;
	s32				level;
	int				limit	= 100000;
	os_time_ns_t	t1;
	os_time_ns_t	t2;

	gsc_label("Time To Clear");
	printf("\n");
	gsc_label_level_inc();

	errs	+= dsi16wrc_initialize		(args->fd, -1, 1);
	errs	+= dsi16wrc_ndiv			(args->fd, -1, 1, 1, NULL);
	errs	+= dsi16wrc_nref			(args->fd, -1, 1, 25, NULL);
	errs	+= dsi16wrc_nvco			(args->fd, -1, 1, 300, NULL);
	errs	+= dsi16wrc_ai_buf_enable	(args->fd, -1, 1, DSI16WRC_AI_BUF_ENABLE_YES, NULL);
	os_sleep_ms(25);
	errs	+= dsi16wrc_ai_buf_fill_lvl	(args->fd, -1, 1, &level);

	errs	+= dsi16wrc_ai_buf_enable	(args->fd, -1, 1, DSI16WRC_AI_BUF_ENABLE_NO, NULL);

	gsc_label("Setting Clear Buffer Bit");
	printf("\n");
	fflush(stdout);
	errs	+= dsi16wrc_reg_mod			(args->fd, -1, 0, DSI16WRC_GSC_BUFCR, D20, D20);

	os_time_get_ns(&t1);

	for (i = 0;;)
	{
		if (i >= limit)
			break;

		errs	+= dsi16wrc_ai_buf_fill_lvl(args->fd, -1, 0, &level);
		i++;

		if (level == 0)
			break;
	}

	os_time_get_ns(&t2);
	errs	+= dsi16wrc_initialize		(args->fd, -1, 1);
	gsc_label("Time To Clear Buffer");

	if (i >= limit)
		printf("Buffer not clear after ");

	delta	= ((long long) t2.tv_sec * 1000000000 + t2.tv_nsec)
			- ((long long) t1.tv_sec * 1000000000 + t1.tv_nsec);

	gsc_label_long_comma(delta);
	printf(" ns, ");
	gsc_label_long_comma(i);
	printf(" register read%s\n", (i == 1) ? "" : "s");

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	ai_buf_clear_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DSI16WRC_IOCTL_AI_BUF_CLEAR.
*
*	Arguments:
*
*		args	The command line argument structure.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai_buf_clear_test(const args_t* args)
{
	int	errs	= 0;

	gsc_label("DSI16WRC_IOCTL_AI_BUF_CLEAR");
	errs	+= _service_test(args);
	errs	+= _function_test(args);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	if (args->verbose)
		errs	+= _timing_test(args);

	return(errs);
}


