// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/sbtest/data_width.c $
// $Rev: 52509 $
// $Date: 2023-03-01 12:40:28 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_DATA_WIDTH,
			/* arg		*/	DSI16WRC_DATA_WIDTH_16,
			/* reg		*/	DSI16WRC_GSC_BUFCR,
			/* mask		*/	0x600000,
			/* value	*/	0x000000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_DATA_WIDTH,
			/* arg		*/	DSI16WRC_DATA_WIDTH_18,
			/* reg		*/	DSI16WRC_GSC_BUFCR,
			/* mask		*/	0x600000,
			/* value	*/	0x200000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_DATA_WIDTH,
			/* arg		*/	DSI16WRC_DATA_WIDTH_20,
			/* reg		*/	DSI16WRC_GSC_BUFCR,
			/* mask		*/	0x600000,
			/* value	*/	0x400000
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	DSI16WRC_IOCTL_DATA_WIDTH,
			/* arg		*/	DSI16WRC_DATA_WIDTH_24,
			/* reg		*/	DSI16WRC_GSC_BUFCR,
			/* mask		*/	0x600000,
			/* value	*/	0x600000
		},

		{ SERVICE_END_LIST }
	};

	int errs	= 0;

	errs	+= dsi16wrc_initialize(fd, -1, 0);
	errs	+= service_ioctl_set_get_list(fd, list);
	errs	+= service_ioctl_set_get_list(fd, list);
	errs	+= dsi16wrc_initialize(fd, -1, 0);
	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	u32	data;
	int	errs	= 0;

	errs	+= dsi16wrc_initialize		(fd, -1, 0);
	errs	+= dsi16wrc_burst			(fd, -1, 0, DSI16WRC_BURST_ENABLE, NULL);
	errs	+= dsi16wrc_burst_size		(fd, -1, 0, 1, NULL);
	errs	+= dsi16wrc_ai_buf_enable	(fd, -1, 0, DSI16WRC_AI_BUF_ENABLE_YES, NULL);
	errs	+= dsi16wrc_ai_mode			(fd, -1, 0, DSI16WRC_AI_MODE_ZERO, NULL);
	errs	+= dsi16wrc_data_format		(fd, -1, 0, DSI16WRC_DATA_FORMAT_OFF_BIN, NULL);

	errs	+= dsi16wrc_ai_buf_clear	(fd, -1, 0);
	errs	+= dsi16wrc_data_width		(fd, -1, 0, DSI16WRC_DATA_WIDTH_16, NULL);
	errs	+= dsi16wrc_burst_trigger	(fd, -1, 0);
	os_sleep_ms(1);
	errs	+= dsi16wrc_reg_read		(fd, -1, 0, DSI16WRC_GSC_IDBR, &data);
	data	&= 0xFFFFF000;

	if ((data != 0x8000) && (data != 0x7000))
		errs++;

	errs	+= dsi16wrc_ai_buf_clear	(fd, -1, 0);
	errs	+= dsi16wrc_data_width		(fd, -1, 0, DSI16WRC_DATA_WIDTH_18, NULL);
	errs	+= dsi16wrc_burst_trigger	(fd, -1, 0);
	os_sleep_ms(1);
	errs	+= dsi16wrc_reg_read		(fd, -1, 0, DSI16WRC_GSC_IDBR, &data);
	data	&= 0xFFFC000;

	if ((data != 0x20000) && (data != 0x1C000))
		errs++;

	errs	+= dsi16wrc_ai_buf_clear	(fd, -1, 0);
	errs	+= dsi16wrc_data_width		(fd, -1, 0, DSI16WRC_DATA_WIDTH_20, NULL);
	errs	+= dsi16wrc_burst_trigger	(fd, -1, 0);
	os_sleep_ms(1);
	errs	+= dsi16wrc_reg_read		(fd, -1, 0, DSI16WRC_GSC_IDBR, &data);
	data	&= 0xFFF0000;

	if ((data != 0x80000) && (data != 0x70000))
		errs++;

	errs	+= dsi16wrc_ai_buf_clear	(fd, -1, 0);
	errs	+= dsi16wrc_data_width		(fd, -1, 0, DSI16WRC_DATA_WIDTH_24, NULL);
	errs	+= dsi16wrc_burst_trigger	(fd, -1, 0);
	os_sleep_ms(1);
	errs	+= dsi16wrc_reg_read		(fd, -1, 0, DSI16WRC_GSC_IDBR, &data);
	data	&= 0xFF00000;

	if ((data != 0x800000) && (data != 0x700000))
		errs++;

	errs	+= dsi16wrc_initialize(fd, -1, 0);
	return(errs);
}



/******************************************************************************
*
*	Function:	data_width_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DSI16WRC_IOCTL_DATA_WIDTH.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int data_width_test(int fd)
{
	int	errs	= 0;

	gsc_label("DSI16WRC_IOCTL_DATA_WIDTH");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}


