// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/sbtest/main.h $
// $Rev: 52509 $
// $Date: 2023-03-01 12:40:28 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <semaphore.h>
#include <time.h>

#include "24dsi16wrc_main.h"



// macros *********************************************************************

#define	SIZEOF_ARRAY(a)			(sizeof((a))/sizeof((a)[0]))



// data types *****************************************************************

typedef struct
{
	// Application Settings

	s32			continuous;	// > 0 = ignore errors, < 0 = stop on errors
	s32			index;
	s32			minute_limit;
	s32			test_limit;
	s32			verbose;

	int			fd;

	// Device Settings

} args_t;

typedef enum
{
	SERVICE_END_LIST,	// Ends list of service_data_t structures.
	SERVICE_NORMAL,
	SERVICE_REG_MOD,
	SERVICE_REG_READ,
	SERVICE_REG_SHOW,
	SERVICE_REG_TEST,
	SERVICE_SLEEP,
	SERVICE_SLEEP_MS,
	SERVICE_IOCTL_GET,
	SERVICE_IOCTL_SET
} service_t;

typedef struct
{
	service_t		service;
	int				cmd;	// IOCTL code
	s32				arg;	// The IOCTL data argument.
	u32				reg;	// The register to access. Use -1 to ignore.
	u32				mask;	// The register bits of interest.
	u32				value;	// The value expected for the bits of interest.
} service_data_t;



// prototypes *****************************************************************


int	adc_mode_test(int fd);					// DSI16WRC_IOCTL_ADC_MODE
int	ai_buf_clear_test(const args_t* args);	// DSI16WRC_IOCTL_AI_BUF_CLEAR
int	ai_buf_enable_test(int fd);				// DSI16WRC_IOCTL_AI_BUF_ENABLE
int	ai_buf_fill_lvl_test(int fd);			// DSI16WRC_IOCTL_AI_BUF_FILL_LVL
int	ai_buf_overflow_test(int fd);			// DSI16WRC_IOCTL_AI_BUF_OVERFLOW
int	ai_buf_thr_sts_test(int fd);			// DSI16WRC_IOCTL_AI_BUF_THR_STS
int	ai_buf_thresh_test(int fd);				// DSI16WRC_IOCTL_AI_BUF_THRESH
int	ai_buf_underflow_test(int fd);			// DSI16WRC_IOCTL_AI_BUF_UNDERFLOW
int	ai_channel_tag_test(int fd);			// DSI16WRC_IOCTL_AI_CHANNEL_TAG
int	ai_mode_test(int fd);					// DSI16WRC_IOCTL_AI_MODE
int	auto_cal_sts_test(int fd);				// DSI16WRC_IOCTL_AUTO_CAL_STS
int	auto_calibrate_test(int fd);			// DSI16WRC_IOCTL_AUTO_CALIBRATE
int	aux_clk_ctl_mode_test(int fd);			// DSI16WRC_IOCTL_AUX_CLK_CTL_MODE
int	aux_sync_ctl_mode_test(int fd);			// DSI16WRC_IOCTL_AUX_SYNC_CTL_MODE

int	burst_rate_div_test(int fd);			// DSI16WRC_IOCTL_BURST_RATE_DIV
int	burst_size_test(int fd);				// DSI16WRC_IOCTL_BURST_SIZE
int	burst_test(int fd);						// DSI16WRC_IOCTL_BURST
int	burst_timer_test(int fd);				// DSI16WRC_IOCTL_BURST_TIMER
int	burst_trigger_test(int fd);				// DSI16WRC_IOCTL_BURST_TRIGGER

int	ch_grp_0_src_test(int fd);				// DSI16WRC_IOCTL_CH_GRP_0_SRC
int	ch_grp_1_src_test(int fd);				// DSI16WRC_IOCTL_CH_GRP_1_SRC
int	channels_ready_test(int fd);			// DSI16WRC_IOCTL_CHANNELS_READY
int	control_mode_test(int fd);				// DSI16WRC_IOCTL_CONTROL_MODE

int	data_format_test(int fd);				// DSI16WRC_IOCTL_DATA_FORMAT
int	data_width_test(int fd);				// DSI16WRC_IOCTL_DATA_WIDTH
int	dio_dir_out_test(int fd);				// DSI16WRC_IOCTL_DIO_DIR_OUT
int	dio_read_test(int fd);					// DSI16WRC_IOCTL_DIO_READ
int	dio_write_test(int fd);					// DSI16WRC_IOCTL_DIO_WRITE

int	ext_clk_src_test(int fd);				// DSI16WRC_IOCTL_EXT_CLK_SRC
int	ext_sync_in_test(int fd);				// DSI16WRC_IOCTL_EXT_SYNC_IN

int	initialize_test(int fd);				// DSI16WRC_IOCTL_INITIALIZE
int	irq_sel_test(int fd);					// DSI16WRC_IOCTL_IRQ_SEL

int	master_clk_adj_test(int fd);			// DSI16WRC_IOCTL_MASTER_CLK_ADJ

int	ndiv_test(int fd);						// DSI16WRC_IOCTL_NDIV
int	nref_test(int fd);						// DSI16WRC_IOCTL_NREF
int	nvco_test(int fd);						// DSI16WRC_IOCTL_NVCO

int	perform_tests(const args_t* args);

int	query_test(int fd);						// DSI16WRC_IOCTL_QUERY

int	range_test(int fd);						// DSI16WRC_IOCTL_RANGE
int	reg_mod_test(int fd);					// DSI16WRC_IOCTL_REG_MOD
int	reg_read_test(int fd);					// DSI16WRC_IOCTL_REG_READ
int	reg_write_test(int fd);					// DSI16WRC_IOCTL_REG_WRITE
int	rx_io_abort_test(int fd);				// DSI16WRC_IOCTL_RX_IO_ABORT
int	rx_io_mode_test(int fd);				// DSI16WRC_IOCTL_RX_IO_MODE
int	rx_io_overflow_test(int fd);			// DSI16WRC_IOCTL_RX_IO_OVERFLOW
int	rx_io_timeout_test(int fd);				// DSI16WRC_IOCTL_RX_IO_TIMEOUT
int	rx_io_underflow_test(int fd);			// DSI16WRC_IOCTL_RX_IO_UNDERFLOW

int	service_ioctl_reg_get_list(int fd, const service_data_t* list);
int	service_ioctl_set_get_list(int fd, const service_data_t* list);
int	service_ioctl_set_reg_list(int fd, const service_data_t* list);
int	sw_sync_mode_test(int fd);				// DSI16WRC_IOCTL_SW_SYNC_MODE
int	sw_sync_test(int fd);					// DSI16WRC_IOCTL_SW_SYNC

int	wait_cancel_test(int fd);				// DSI16WRC_IOCTL_WAIT_CANCEL
int	wait_event_test(int fd);				// DSI16WRC_IOCTL_WAIT_EVENT
int	wait_status_test(int fd);				// DSI16WRC_IOCTL_WAIT_STATUS

int	xcvr_type_test(int fd);					// DSI16WRC_IOCTL_XCVR_TYPE



#endif
