// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/sbtest/reg_mod.c $
// $Rev: 52509 $
// $Date: 2023-03-01 12:40:28 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	mask;
	} list[]	=
	{
		// reg					mask
		{ DSI16WRC_GSC_BCTLR,	0x0000073F	},
		{ DSI16WRC_GSC_RDR,		0x000001FF	},
		{ DSI16WRC_GSC_BUFCR,	0x0003FFFF	},
	};

	u32			bit;
	int			errs	= 0;
	int			i;
	int			j;
	gsc_reg_t	parm;
	u32			value;

	for (i = 0; (errs == 0) && (i < (int) SIZEOF_ARRAY(list)); i++)
	{

		for (bit = 0x1; bit; bit <<= 1)
		{
			if ((list[i].mask & bit) == 0)
				continue;

			// Read the register value so we can restore it later.
			parm.reg	= list[i].reg;
			j			= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_REG_READ, (void*) &parm);
			value		= parm.value;

			if (j)
			{
				errs	= 1;
				break;
			}

			// Set the bit low.
			parm.value	= 0;
			parm.mask	= bit;
			j			= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_REG_MOD, (void*) &parm);

			if (j)
			{
				errs	= 1;
				break;
			}

			// Now verify that the bit is low.
			j	= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_REG_READ, (void*) &parm);

			if (j)
			{
				errs	= 1;
				break;
			}

			if (parm.value & bit)
			{
				errs	= 1;
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			j			= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_REG_WRITE, (void*) &parm);

			if (j)
			{
				errs	= 1;
				break;
			}

			// Set the bit high.
			parm.value	= bit;
			parm.mask	= bit;
			j			= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_REG_MOD, (void*) &parm);

			if (j)
			{
				errs	= 1;
				break;
			}

			// Now verify that the bit is high.
			j	= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_REG_READ, (void*) &parm);

			if (j)
			{
				errs	= 1;
				break;
			}

			if ((parm.value & bit) == 0)
			{
				errs	= 1;
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			j			= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_REG_WRITE, (void*) &parm);

			if (j)
			{
				errs	= 1;
				break;
			}
		}
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;

	return(errs);
}



/******************************************************************************
*
*	Function:	reg_mod_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DSI16WRC_IOCTL_REG_MOD.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_mod_test(int fd)
{
	int	errs	= 0;

	gsc_label("DSI16WRC_IOCTL_REG_MOD");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}


