// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/samples/sbtest/wait_event.c $
// $Rev: 52509 $
// $Date: 2023-03-01 12:40:28 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	_1_SEC		(1000L)
#define	_15_SEC		(15L * 1000L)
#define	_30_SEC		(30L * 1000L)



// variables ******************************************************************

static	int			rx_stat;
static	sem_t		sem_enter;
static	sem_t		sem_exit;
static	gsc_wait_t	wait_dst;
static	int			wait_fd;



//*****************************************************************************
static int _service_test(int fd)
{
	return(0);
}



//*****************************************************************************
static int	_wait_main_dma(int fd, int supported)
{
	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		errs	+= dsi16wrc_rx_io_mode		(fd, -1, 0, GSC_IO_MODE_BMDMA, NULL);
		errs	+= dsi16wrc_ndiv			(fd, -1, 0, 1, NULL);
		errs	+= dsi16wrc_nref			(fd, -1, 0, 25, NULL);
		errs	+= dsi16wrc_nvco			(fd, -1, 0, 300, NULL);
		errs	+= dsi16wrc_ai_buf_enable	(fd, -1, 0, DSI16WRC_AI_BUF_ENABLE_YES, NULL);
		os_sleep_ms(100);
		ret		= dsi16wrc_read(fd, buf, sizeof(buf));
		errs	+= (ret == sizeof(buf)) ? 0 : 1;
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_main_gsc(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize	(fd, -1, 0);
		errs	+= dsi16wrc_irq_sel		(fd, -1, 0, DSI16WRC_IRQ_CHAN_READY, NULL);
		errs	+= dsi16wrc_ai_buf_clear(fd, -1, 0);
		errs	+= dsi16wrc_initialize	(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_init_done(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize	(fd, -1, 0);
		errs	+= dsi16wrc_irq_sel		(fd, -1, 0, DSI16WRC_WAIT_GSC_INIT_DONE, NULL);
		errs	+= dsi16wrc_initialize	(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_auto_cal_done(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		errs	+= dsi16wrc_irq_sel			(fd, -1, 0, DSI16WRC_IRQ_AUTO_CAL_DONE, NULL);
		errs	+= dsi16wrc_auto_calibrate	(fd, -1, 0);
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_channel_ready(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize	(fd, -1, 0);
		errs	+= dsi16wrc_irq_sel		(fd, -1, 0, DSI16WRC_IRQ_CHAN_READY, NULL);
		errs	+= dsi16wrc_adc_mode	(fd, -1, 0, DSI16WRC_ADC_MODE_HI_SPEED, NULL);
		os_sleep_ms(10);
		errs	+= dsi16wrc_initialize	(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_buf_thresh_l2h(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		errs	+= dsi16wrc_irq_sel			(fd, -1, 0, DSI16WRC_IRQ_AI_BUF_THRESH_L2H, NULL);
		errs	+= dsi16wrc_ai_buf_thresh	(fd, -1, 0, 32, NULL);
		errs	+= dsi16wrc_ai_buf_enable	(fd, -1, 0, DSI16WRC_AI_BUF_ENABLE_YES, NULL);
		os_sleep_ms(10);
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_buf_thresh_h2l(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		errs	+= dsi16wrc_irq_sel			(fd, -1, 0, DSI16WRC_IRQ_AI_BUF_THRESH_H2L, NULL);
		errs	+= dsi16wrc_ndiv			(fd, -1, 0, 1, NULL);
		errs	+= dsi16wrc_nref			(fd, -1, 0, 25, NULL);
		errs	+= dsi16wrc_nvco			(fd, -1, 0, 300, NULL);
		errs	+= dsi16wrc_ai_buf_thresh	(fd, -1, 0, 32, NULL);
		errs	+= dsi16wrc_ai_buf_enable	(fd, -1, 0, DSI16WRC_AI_BUF_ENABLE_YES, NULL);
		os_sleep_ms(10);
		errs	+= dsi16wrc_ai_buf_enable	(fd, -1, 0, DSI16WRC_AI_BUF_ENABLE_NO, NULL);
		errs	+= dsi16wrc_ai_buf_clear	(fd, -1, 0);
		os_sleep_ms(10);
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_gsc_burst_done(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		errs	+= dsi16wrc_irq_sel			(fd, -1, 0, DSI16WRC_IRQ_AI_BURST_DONE, NULL);
		errs	+= dsi16wrc_burst			(fd, -1, 0, DSI16WRC_BURST_ENABLE, NULL);
		errs	+= dsi16wrc_burst_size		(fd, -1, 0, 1, NULL);
		errs	+= dsi16wrc_ai_buf_enable	(fd, -1, 0, DSI16WRC_AI_BUF_ENABLE_YES, NULL);
		errs	+= dsi16wrc_burst_trigger	(fd, -1, 0);
		os_sleep_ms(10);
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_rx_done(int fd, int supported)
{
	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		errs	+= dsi16wrc_ndiv			(fd, -1, 0, 1, NULL);
		errs	+= dsi16wrc_nref			(fd, -1, 0, 25, NULL);
		errs	+= dsi16wrc_nvco			(fd, -1, 0, 300, NULL);
		errs	+= dsi16wrc_ai_buf_enable	(fd, -1, 0, DSI16WRC_AI_BUF_ENABLE_YES, NULL);
		os_sleep_ms(100);
		ret		= dsi16wrc_read(fd, buf, sizeof(buf));
		errs	+= (ret == sizeof(buf)) ? 0 : 1;
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_rx_error(int fd, int supported)
{
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		// Now perform a read to trigger the wait.
		ret	= dsi16wrc_read(fd, NULL, 3);

		if (ret >= 0)
			errs++;

		ret	= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_io_rx_timeout(int fd, int supported)
{
	u32	buf[1024];
	int	errs	= 0;
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		errs	+= dsi16wrc_rx_io_timeout	(fd, -1, 0, 1, NULL);
		errs	+= dsi16wrc_ai_buf_enable	(fd, -1, 0, DSI16WRC_AI_BUF_ENABLE_NO, NULL);
		errs	+= dsi16wrc_ai_buf_clear	(fd, -1, 0);
		ret		= dsi16wrc_read(fd, buf, sizeof(buf));
		errs	+= (ret == 0) ? 0 : 1;
		errs	+= dsi16wrc_initialize(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int _read_thread(void* arg)
{
	u32		buf[1024];
	int		fd	= (int) (long) arg;
	long	status;

	status	= dsi16wrc_read(fd, buf, sizeof(buf));

	if (status == 0)
	{
		rx_stat	= 1;
	}
	else
	{
		rx_stat	= 2;
		printf("FAIL <---  (%d. dsi16wrc_read returned %ld, expected 0)\n",
				__LINE__,
				(long) status);
	}

	return(0);
}



//*****************************************************************************
static int	_wait_io_rx_abort(int fd, int supported)
{
	int			errs	= 0;
	int			i;
	int			ret;
	s32			set;
	os_thread_t	thread;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		errs	+= dsi16wrc_initialize		(fd, -1, 0);
		errs	+= dsi16wrc_rx_io_timeout	(fd, -1, 0, 10, NULL);

		// Now start the read thread so we can do the abort.
		memset(&thread, 0, sizeof(thread));

		if (errs == 0)
		{
			errs	+= os_thread_create(	&thread,
											"Rx",
											_read_thread,
											(void*) (long) fd);
		}

		for (i = 0; (errs == 0) && (i < 1000); i++)
		{
			set		= 0;
			ret		= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_RX_IO_ABORT, &set);
			errs	+= ret ? 1 : 0;

			if (errs)
				break;

			if (set)
				break;

			os_sleep_ms(10);
		}

		// Wait for the thread to report its status.

		for (i = 0; i < 1000; i++)
		{
			if (errs)
				break;

			if (rx_stat == 0)
			{
				os_sleep_ms(10);
				continue;
			}

			if (rx_stat == 1)
				break;

			errs++;
			break;
		}

		errs	+= os_thread_destroy(&thread);
		errs	+= dsi16wrc_initialize(fd, -1, 0);
		ret		= errs;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_timeout(int fd, int supported)
{
	int	ret;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		gsc_time_sleep_ms(2000);
		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
static int	_wait_cancel(int fd, int supported)
{
	int			errs	= 0;
	int			ret;
	gsc_wait_t	wait;

	if (supported)
	{
		ret	= -1;
	}
	else
	{
		wait.flags			= 0;
		wait.main			= 0xFFFFFFFF;
		wait.gsc			= 0xFFFFFFFF;
		wait.alt			= 0xFFFFFFFF;
		wait.io				= 0xFFFFFFFF;
		wait.timeout_ms		= 0;
		wait.count			= 0;
		ret		= dsi16wrc_ioctl(wait_fd, DSI16WRC_IOCTL_WAIT_CANCEL, &wait);
		errs	+= ret ? 1 : 0;
		ret		= errs;
	}

	return(ret);
}



// variables ******************************************************************

static const struct
{
	const char*	name;
	int			(*function)(int fd, int supported);
	gsc_wait_t	wait;
} wait_src[]	=
{
	// name						function					flags			main									...			timeout_ms	count
	{ "DMA0/DMA1",				_wait_main_dma,				{ 0,			GSC_WAIT_MAIN_DMA0 |
																			GSC_WAIT_MAIN_DMA1,						0, 0, 0,	_15_SEC,	0	}	},
	{ "GSC",					_wait_main_gsc,				{ 0,			GSC_WAIT_MAIN_GSC,						0, 0, 0,	_15_SEC,	0	}	},

	// name						function					flags ...		gsc										...			timeout_ms
	{ "Initialization Done",	_wait_gsc_init_done,		{ 0, 0,			DSI16WRC_WAIT_GSC_INIT_DONE,			0, 0,		_15_SEC,	0	}	},
	{ "Auto-Cal Done",			_wait_gsc_auto_cal_done,	{ 0, 0,			DSI16WRC_WAIT_GSC_AUTO_CAL_DONE,		0, 0,		_15_SEC,	0	}	},
	{ "Channel Ready",			_wait_gsc_channel_ready,	{ 0, 0,			DSI16WRC_WAIT_GSC_CHAN_READY,			0, 0,		_15_SEC,	0	}	},
	{ "Buf Threshold Low->Hi",	_wait_gsc_buf_thresh_l2h,	{ 0, 0,			DSI16WRC_WAIT_GSC_AI_BUF_THRESH_L2H,	0, 0,		_15_SEC,	0	}	},
	{ "Buf Threshold Hi->Low",	_wait_gsc_buf_thresh_h2l,	{ 0, 0,			DSI16WRC_WAIT_GSC_AI_BUF_THRESH_H2L,	0, 0,		_15_SEC,	0	}	},
	{ "Burst Done",				_wait_gsc_burst_done,		{ 0, 0,			DSI16WRC_WAIT_GSC_AI_BURST_DONE,		0, 0,		_15_SEC,	0	}	},


	// name						function					flags ...		io													timeout_ms
	{ "Rx Done",				_wait_io_rx_done,			{ 0, 0, 0, 0,	DSI16WRC_WAIT_IO_RX_DONE,							_15_SEC,	0	}	},
	{ "Rx Error",				_wait_io_rx_error,			{ 0, 0, 0, 0,	DSI16WRC_WAIT_IO_RX_ERROR,							_15_SEC,	0	}	},
	{ "Rx Timeout",				_wait_io_rx_timeout,		{ 0, 0, 0, 0,	DSI16WRC_WAIT_IO_RX_TIMEOUT,						_15_SEC,	0	}	},
	{ "Rx Abort",				_wait_io_rx_abort,			{ 0, 0, 0, 0,	DSI16WRC_WAIT_IO_RX_ABORT,							_15_SEC,	0	}	},

	// name						function					flags ...		io													timeout_ms
	{ "Wait Timeout",			_wait_timeout,				{ 0, 0, 0, 0,	DSI16WRC_WAIT_IO_RX_DONE,							_1_SEC,		0	}	},
	{ "Wait Cancel",			_wait_cancel,				{ 0, 0, 0, 0,	DSI16WRC_WAIT_IO_RX_DONE,							_30_SEC,	0	}	},

	// terminate list
	{ NULL, NULL, { 1, 0, 0, 0,	0, 0, 0	} }
};



//*****************************************************************************
static int _wait_thread(void* arg)
{
	int	status;

	sem_post(&sem_enter);
	status	= dsi16wrc_ioctl(wait_fd, DSI16WRC_IOCTL_WAIT_EVENT, &wait_dst);

	if (status == -1)
		wait_dst.flags	= 0xFFFFFFFF;

	sem_post(&sem_exit);
	return(0);
}



//*****************************************************************************
static int _process_thread(int index)
{
	char		buf[64];
	int			errs	= 0;
	int			i;
	int			ret;
	int			status;
	os_thread_t	thread;
	gsc_wait_t	wait;

	for (;;)	// A convenience loop.
	{
		sem_init(&sem_enter, 0, 0);
		sem_init(&sem_exit, 0, 0);
		memset(&thread, 0, sizeof(thread));
		sprintf(buf, "wait event %d", index);
		wait_dst	= wait_src[index].wait;
		errs		+= os_thread_create(&thread, buf, _wait_thread, NULL);

		// Wait for the thread to become waiting.
		sem_wait(&sem_enter);

		if (errs == 0)
		{
			wait	= wait_src[index].wait;

			for (i = 0; i < 100; i++)
			{
				status		= dsi16wrc_ioctl(wait_fd, DSI16WRC_IOCTL_WAIT_STATUS, &wait);

				if (status == -1)
				{
					printf(	"FAIL <---  (%d. status request %d: errno %d)\n",
							__LINE__,
							index,
							errno);
					errs++;
					break;
				}

				if (wait.count == 0)
				{
					gsc_time_sleep_ms(100);
					continue;
				}

				if (wait.count == 1)
					break;

				errs++;
				printf(	"FAIL <---  (%d. invalid wait count %ld)\n",
						__LINE__,
						(long) wait.count);
				break;
			}
		}

		// Initiate the respective event.
		errs	+= wait_src[index].function(wait_fd, 0);

		if (errs)
			break;		// There was an error.

		// Verify that the wait ended as expected.

		if (wait_src[index].wait.flags == 0xFFFFFFFF)
			break;		// There was an error.

		// Wait for the resumed thread to run.
		sem_wait(&sem_exit);

		for (i = 0;; i++)
		{
			if (wait_dst.flags)
				break;

			if (i >= 100)
			{
				printf(	"FAIL <---  (%d. thread failed to resume)\n",
						__LINE__);
				errs++;
				break;
			}

			gsc_time_sleep_ms(100);
		}

		if (errs)
			break;		// There was an error.

		if (wait_src[index].wait.timeout_ms == _1_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_TIMEOUT)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_TIMEOUT,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		if (wait_src[index].wait.timeout_ms == _15_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_DONE)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_DONE,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		if (wait_src[index].wait.timeout_ms == _30_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_CANCEL)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_TIMEOUT,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		printf("FAIL <---  (%d. INTERNAL ERROR)\n", __LINE__);
		errs++;
		break;
	}

	// Force termination, just in case of an error.
	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= dsi16wrc_ioctl(wait_fd, DSI16WRC_IOCTL_WAIT_CANCEL, &wait);
	errs	+= ret ? 1 : 0;
	errs	+= os_thread_destroy(&thread);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	int	errs	= 0;
	int	i;
	int	ret;

	printf("\n");
	gsc_label_level_inc();

	// Create a thread awaiting each of the source crtiteria
	// and verify that it resumes when the criteria is met.

	for (i = 0; wait_src[i].wait.flags == 0; i++)
	{
		gsc_label(wait_src[i].name);
		ret	= wait_src[i].function(wait_fd, 1);

		if (ret > 0)
		{
		}
		else if (ret == 0)
		{
			printf("SKIPPED  (Not supported on this device.)\n");
		}
		else
		{
			wait_fd	= fd;
			memset(&wait_dst, 0, sizeof(wait_dst));
			errs	+= _process_thread(i);
		}
	}

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	wait_event_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service DSI16WRC_IOCTL_WAIT_EVENT.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int wait_event_test(int fd)
{
	int	errs	= 0;

	gsc_label("DSI16WRC_IOCTL_WAIT_EVENT");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	return(errs);
}


