// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/utils/util_aux_clk_ctl_mode.c $
// $Rev: 44343 $
// $Date: 2019-01-28 15:13:50 -0600 (Mon, 28 Jan 2019) $

// 24DSI16WRC: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi16wrc_aux_clk_ctl_mode
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI16WRC_IOCTL_AUX_CLK_CTL_MODE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi16wrc_aux_clk_ctl_mode(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("Aux Clock Control Mode", index);

	ret		= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_AUX_CLK_CTL_MODE, &set);
	errs	+= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI16WRC_AUX_CLK_CTL_MODE_INACTIVE:

			ptr	= "Inactive";
			break;

		case DSI16WRC_AUX_CLK_CTL_MODE_INPUT:

			ptr	= "Input";
			break;

		case DSI16WRC_AUX_CLK_CTL_MODE_OUTPUT:

			ptr	= "Output";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


