// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/utils/util_channels_ready.c $
// $Rev: 42850 $
// $Date: 2018-05-18 10:42:17 -0500 (Fri, 18 May 2018) $

// 24DSI16WRC: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi16wrc_channels_ready
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI16WRC_IOCTL_CHANNELS_READY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi16wrc_channels_ready(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr		= NULL;
	int			ret;

	if (verbose)
		gsc_label_index("Channels Ready", index);

	ret	= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_CHANNELS_READY, (void*) &set);

	if (ret)
	{
		if (ret == -ETIMEDOUT)
		{
			set	= DSI16WRC_CHANNELS_READY_NO;
		}
		else
		{
			errs++;
			ptr	= buf;
			sprintf(buf, "ERROR: dsi16wrc_ioctl() failure, errno = %d\n", errno);
		}
	}

	switch (set)
	{
		default:

			errs++;
			ptr	= ptr ? ptr : buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI16WRC_CHANNELS_READY_NO:

			ptr	= ptr ? ptr : "No, timed out";
			break;

		case DSI16WRC_CHANNELS_READY_YES:

			ptr	= ptr ? ptr : "Yes";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


