// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/utils/util_config_ai.c $
// $Rev: 52506 $
// $Date: 2023-03-01 12:36:56 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi16wrc_config_ai
*
*	Purpose:
*
*		Configure the given device using common defaults.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fref	This is the PLL Fref value, or -1 to use the default.
*
*		fsamp	This is the desired Fsamp rate, of -1 to use the default.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int	dsi16wrc_config_ai(int fd, int index, int verbose, s32 fref, s32 fsamp)
{
	int	errs	= 0;
	s32	mca;
	s32	mode;
	s32	ndiv;
	s32	nref;
	s32	nvco;

	if (verbose)
	{
		gsc_label_index("Input Configuration", index);
		gsc_label_level_inc();
		printf("\n");
	}

	errs	+= dsi16wrc_initialize			(fd, index, verbose);
	errs	+= dsi16wrc_rx_io_mode			(fd, index, verbose, GSC_IO_MODE_BMDMA,						NULL);
	errs	+= dsi16wrc_rx_io_overflow		(fd, index, verbose, DSI16WRC_IO_OVERFLOW_CHECK,			NULL);
	errs	+= dsi16wrc_rx_io_timeout		(fd, index, verbose, 30,									NULL);
	errs	+= dsi16wrc_rx_io_underflow		(fd, index, verbose, DSI16WRC_IO_UNDERFLOW_CHECK,			NULL);
	errs	+= dsi16wrc_ai_mode				(fd, index, verbose, DSI16WRC_AI_MODE_DIFF,					NULL);
	errs	+= dsi16wrc_ai_buf_thresh		(fd, index, verbose, 128L * 1024,							NULL);
	errs	+= dsi16wrc_ai_channel_tag		(fd, index, verbose, DSI16WRC_AI_CHANNEL_TAG_ENABLE,		NULL);
	errs	+= dsi16wrc_sw_sync_mode		(fd, index, verbose, DSI16WRC_SW_SYNC_MODE_CLR_BUF,			NULL);
	errs	+= dsi16wrc_data_format			(fd, index, verbose, DSI16WRC_DATA_FORMAT_OFF_BIN,			NULL);
	errs	+= dsi16wrc_data_width			(fd, index, verbose, DSI16WRC_DATA_WIDTH_16,				NULL);
	errs	+= dsi16wrc_dio_dir_out			(fd, index, verbose, 0x0,									NULL);
	errs	+= dsi16wrc_control_mode		(fd, index, verbose, DSI16WRC_CONTROL_MODE_INITIATOR,		NULL);
	errs	+= dsi16wrc_ext_clk_src			(fd, index, verbose, DSI16WRC_EXT_CLK_SRC_GRP_0,			NULL);
	errs	+= dsi16wrc_ext_sync_in			(fd, index, verbose, DSI16WRC_EXT_SYNC_IN_DISABLE,			NULL);
	errs	+= dsi16wrc_irq_sel				(fd, index, verbose, DSI16WRC_IRQ_INIT_DONE,				NULL);
	errs	+= dsi16wrc_ch_grp_0_src		(fd, index, verbose, DSI16WRC_CH_GRP_SRC_RATE_GEN,			NULL);
	errs	+= dsi16wrc_ch_grp_1_src		(fd, index, verbose, DSI16WRC_CH_GRP_SRC_RATE_GEN,			NULL);
	errs	+= dsi16wrc_range				(fd, index, verbose, DSI16WRC_RANGE_10V,					NULL);
	errs	+= dsi16wrc_query				(fd, index, verbose, DSI16WRC_QUERY_FREF_DEFAULT, &fref);
	errs	+= dsi16wrc_fsamp_ai_compute	(fd, index, verbose, fref, fsamp, &mode, &nvco, &nref, &ndiv, &mca, NULL);
	errs	+= dsi16wrc_master_clk_adj		(fd, index, verbose, mca,									NULL);
	errs	+= dsi16wrc_adc_mode			(fd, index, verbose, mode,									NULL);
	errs	+= dsi16wrc_nvco				(fd, index, verbose, nvco,									NULL);
	errs	+= dsi16wrc_nref				(fd, index, verbose, nref,									NULL);
	errs	+= dsi16wrc_ndiv				(fd, index, verbose, ndiv,									NULL);
	errs	+= dsi16wrc_fsamp_ai_report_all	(fd, index, verbose,										NULL);
	errs	+= dsi16wrc_xcvr_type			(fd, index, verbose, DSI16WRC_XCVR_TYPE_LVDS,				NULL);
	errs	+= dsi16wrc_aux_clk_ctl_mode	(fd, index, verbose, DSI16WRC_AUX_CLK_CTL_MODE_INACTIVE,	NULL);
	errs	+= dsi16wrc_aux_sync_ctl_mode	(fd, index, verbose, DSI16WRC_AUX_SYNC_CTL_MODE_INACTIVE,	NULL);
	errs	+= dsi16wrc_burst				(fd, index, verbose, DSI16WRC_BURST_DISABLE,				NULL);
	errs	+= dsi16wrc_burst_rate_div		(fd, index, verbose, 1,										NULL);
	errs	+= dsi16wrc_burst_size			(fd, index, verbose, 1,										NULL);
	errs	+= dsi16wrc_burst_timer			(fd, index, verbose, DSI16WRC_BURST_TIMER_DISABLE,			NULL);
	errs	+= dsi16wrc_ai_buf_enable		(fd, index, verbose, DSI16WRC_AI_BUF_ENABLE_YES,			NULL);
	errs	+= dsi16wrc_auto_calibrate		(fd, index, verbose);	// Cable may need disconnecting for this.
	errs	+= dsi16wrc_auto_cal_sts		(fd, index, verbose,										NULL);
	errs	+= dsi16wrc_ai_buf_clear		(fd, index, verbose);
	errs	+= dsi16wrc_ai_buf_overflow		(fd, index, verbose, DSI16WRC_AI_BUF_OVERFLOW_TEST,			NULL);
	errs	+= dsi16wrc_ai_buf_underflow	(fd, index, verbose, DSI16WRC_AI_BUF_UNDERFLOW_TEST,		NULL);

	if (verbose)
		gsc_label_level_dec();

	return(errs);
}


