// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/utils/util_ext_sync_in.c $
// $Rev: 52506 $
// $Date: 2023-03-01 12:36:56 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: specific utility module.

#include "main.h"



/******************************************************************************
*
*	Function:	dsi16wrc_ext_sync_in
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI16WRC_IOCTL_EXT_SYNC_IN service.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	0: errors only, !0: all output
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi16wrc_ext_sync_in(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	s32			ret;

	if (verbose)
		gsc_label_index("External SYNC Input", index);

	ret		= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_EXT_SYNC_IN, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs	= 1;
			ptr		= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case DSI16WRC_EXT_SYNC_IN_DISABLE:

			ptr	= "Disable";
			break;

		case DSI16WRC_EXT_SYNC_IN_ENABLE:

			ptr	= "Enable";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


