// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/utils/util_fsamp_ai.c $
// $Rev: 52506 $
// $Date: 2023-03-01 12:36:56 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	ABS(v)					(((v) < 0) ? (-(v)) : (v))
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



// data types *****************************************************************

typedef struct
{
	s32		fsamp_want;
	double	fsamp_got;

	double	delta;
	long	divisor;
	s32		fgen_max;
	s32		fgen_min;
	s32		fref;
	s32		fsamp_max;
	s32		fsamp_min;
	s32		mca;		// Master Clock Adjust
	s32		mode;
	s32		ndiv;
	s32		ndiv_max;
	s32		ndiv_min;
	s32		nref;
	s32		nref_max;
	s32		nref_min;
	s32		nvco;
	s32		nvco_max;
	s32		nvco_min;
} fsamp_t;



//*****************************************************************************
static void _dsi16wrc_fsamp_compute_mca(fsamp_t* data)
{
	double	delta;
	s32		div;
	s32		div_max;
	s32		div_min;
	double	fgen;
	s32		fgen_max	= data->fgen_max;
	s32		fgen_min	= data->fgen_min;
	double	fref;
	double	fsamp_want	= (double) data->fsamp_want;
	double	mca;
	s32		ndiv_max	= data->ndiv_max;
	s32		ndiv_min	= data->ndiv_min;
	s32		nref_max	= data->nref_max;
	s32		nref_min	= data->nref_min;
	s32		nvco_max	= data->nvco_max;
	s32		nvco_min	= data->nvco_min;
	s32		ref;
	s32		ref_max;
	s32		ref_min;
	double	fsamp_got;
	s32		vco;
	s32		vco_max;
	s32		vco_min;

	if (data->mca >= 0x8000)
		mca	= + (((double) (data->mca - 0x8000)) / ((double) (0xFFFF - 0x8000))) * 80;
	else
		mca	= - (((double) (0x8000 - data->mca)) / ((double) (0x8000 - 0x0000))) * 80;

	fref	= mca + (double) data->fref;

	div_max	= (s32) ((double) data->fgen_max / ((double) data->fsamp_want * data->divisor));
	div_min	= (s32) ((double) data->fgen_min / ((double) data->fsamp_want * data->divisor));
	div_max	= LIMIT_RANGE(div_max, data->ndiv_min, data->ndiv_max);
	div_min	= LIMIT_RANGE(div_min, data->ndiv_min, data->ndiv_max);

	for (div = div_min - 1; div <= (div_max + 1); div++)
	{
		if ((div < ndiv_min) || (div > ndiv_max))
			continue;

		fgen	= fsamp_want * div * data->divisor;

		if ((fgen < fgen_min) || (fgen > fgen_max))
			continue;

		ref_max	= (s32) (fref * nvco_max / fgen);
		ref_min	= (s32) (fref * nvco_min / fgen);
		ref_max	= LIMIT_RANGE(ref_max, nref_min, nref_max);
		ref_min	= LIMIT_RANGE(ref_min, nref_min, nref_max);

		for (ref = ref_min - 1; ref <= (ref_max + 1); ref++)
		{
			if ((ref < nref_min) || (ref > nref_max))
				continue;

			vco		= (s32) (fgen * ref / fref);
			vco_max	= vco;
			vco_min	= vco;

			for (vco = vco_min - 1; vco <= (vco_max + 1); vco++)
			{
				if ((vco < nvco_min) || (vco > nvco_max))
					continue;

				fgen		= fref * vco / ref;
				fsamp_got	= fgen / div / data->divisor;
				delta		= fsamp_got - fsamp_want;

				if (ABS(delta) < ABS(data->delta))
				{
					data->fsamp_got	= fsamp_got;
					data->delta		= delta;
					data->nvco		= vco;
					data->nref		= ref;
					data->ndiv		= div;
				}
			}
		}
	}
}



//*****************************************************************************
static void _dsi16wrc_fsamp_compute_w_limits(fsamp_t* data)
{
	double	delta;
	double	fgen;
	double	fref;
	int		i;
	s32		mca;
	fsamp_t	tmp;

	for (i = 0; i < 4; i++)
	{
		if (i == 0)
		{
			data->mca	= 0x8000;
			_dsi16wrc_fsamp_compute_mca(data);
		}
		else
		{
			fgen	= (double) data->fsamp_want * data->ndiv * data->divisor;
			fref	= fgen * data->nref / data->nvco;
			delta	= fref - data->fref;
			delta	= LIMIT_RANGE(delta, -80, 80);
			mca		= (double) 0xFFFF / 160 * delta + 0x8000;
			mca		= LIMIT_RANGE(mca, 0x0000, 0xFFFF);
			tmp		= data[0];
			tmp.mca	= mca;
			_dsi16wrc_fsamp_compute_mca(&tmp);

			if (ABS(tmp.delta) < ABS(data->delta))
				data[0]	= tmp;
		}

		if (data->delta == 0)
			break;
	}
}



//*****************************************************************************
static void _dsi16wrc_fsamp_compute_hi_res(fsamp_t* data)
{
	data->mode		= DSI16WRC_ADC_MODE_HI_RES;
	data->fsamp_min	= 200;
	data->fsamp_max	= 52500L;
	data->divisor	= 1024;
	_dsi16wrc_fsamp_compute_w_limits(data);
}



//*****************************************************************************
static void _dsi16wrc_fsamp_compute_hi_speed(fsamp_t* data)
{
	data->mode		= DSI16WRC_ADC_MODE_HI_SPEED;
	data->fsamp_min	= 400;
	data->fsamp_max	= 105000L;
	data->divisor	= 512;
	_dsi16wrc_fsamp_compute_w_limits(data);
}



//*****************************************************************************
static void _dsi16wrc_fsamp_compute_pll(fsamp_t* data)
{
	fsamp_t	res;
	fsamp_t	speed;

	if (data->fsamp_want < 400)
	{
		_dsi16wrc_fsamp_compute_hi_res(data);
	}
	else if (data->fsamp_want > 52500)
	{
		_dsi16wrc_fsamp_compute_hi_speed(data);
	}
	else
	{
		res		= data[0];
		_dsi16wrc_fsamp_compute_hi_res(&res);

		speed	= data[0];
		_dsi16wrc_fsamp_compute_hi_speed(&speed);

		if (ABS(res.delta) <= ABS(speed.delta))
			data[0]	= res;
		else
			data[0]	= speed;
	}
}



//*****************************************************************************
static int _fsamp_report_rate_gen(
	int		verbose,
	s32*	sps,
	s32		fref,
	s32		mca,
	s32		mode,
	s32		ndiv,
	s32		nref,
	s32		nvco)
{
	int		errs	= 0;
	s32		divisor	= (mode == DSI16WRC_ADC_MODE_HI_RES) ? 1024 : 512;
	double	fgen;
	s32		fsamp;
	double	rate;
	double	ref;

	if (mca >= 0x8000)
		ref	= + (((double) (mca - 0x8000)) / ((double) (0xFFFF - 0x8000))) * 80;
	else
		ref	= - (((double) (0x8000 - mca)) / ((double) (0x8000 - 0x0000))) * 80;

	ref		+= fref;
	fgen	= ref * nvco / nref;
	rate	= fgen / ndiv / divisor;
	rate	+= (rate >= 0) ? +0.5 : -0.5;
	fsamp	= (s32) rate;

	if (verbose)
	{
		gsc_label_long_comma((long) fsamp);
		printf(" S/S  (Fref ");
		gsc_label_long_comma((long) fref);
		printf(", MCA 0x%lX", (long) mca);
		printf(", %s", (mode == DSI16WRC_ADC_MODE_HI_RES) ? "Hi Res" : "Hi Speed");
		printf(", Nvco %ld", (long) nvco);
		printf(", Nref %ld", (long) nref);
		printf(", Ndiv %ld)\n", (long) ndiv);
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



//*****************************************************************************
static int _fsamp_report_grp_src(
	int		verbose,
	s32*	sps,
	s32		g_src,
	s32		fref,
	s32		mca,
	s32		mode,
	s32		ndiv,
	s32		nref,
	s32		nvco)
{
	int	errs	= 0;
	s32	fsamp;

	switch (g_src)
	{
		default:

			fsamp	= 0;
			errs++;

			if (verbose)
				printf("FAIL <---  (Unrecodnized source option: %ld)\n", (long) g_src);

			break;

		case DSI16WRC_CH_GRP_SRC_RATE_GEN:

			errs	+= _fsamp_report_rate_gen(verbose, &fsamp, fref, mca, mode, ndiv, nref, nvco);
			break;

		case DSI16WRC_CH_GRP_SRC_EXTERN:
		case DSI16WRC_CH_GRP_SRC_DIR_EXTERN:

			fsamp	= 0;

			if (verbose)
				printf("0 S/S  (External Source)\n");

			break;

		case DSI16WRC_CH_GRP_SRC_DISABLE:

			fsamp	= 0;

			if (verbose)
				printf("0 S/S  (Disabled)\n");

			break;
	}

	if (sps)
		sps[0]	= fsamp;

	return(errs);
}



/******************************************************************************
*
*	Function:	dsi16wrc_fsamp_ai_compute
*
*	Purpose:
*
*		Calculate the best values for the specified sample rate.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fref	The is the Fref value.
*
*		fsamp	The desired sample rate.
*
*		mode	Put the computed mode here. (Hi Speed vd Hi Res)
*
*		nvco	Put the resulting Nvco value here.
*
*		nref	Put the resulting Nref value here.
*
*		ndiv	Put the resulting Ndiv value here.
*
*		mca		Put the Master Clock Adjust value here.
*
*		rate	Report the sample rate achieved here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi16wrc_fsamp_ai_compute(
	int		fd,
	int		index,
	int		verbose,
	s32		fref,
	s32		fsamp,
	s32*	mode,
	s32*	nvco,
	s32*	nref,
	s32*	ndiv,
	s32*	mca,
	double*	rate)
{
	fsamp_t	data;
	int		errs	= 0;
	char*	psz;

	if (verbose)
	{
		gsc_label_index("AI Sample Rate", index);
		gsc_label_long_comma(fsamp);
		printf(" S/S");
	}

	memset(&data, 0, sizeof(data));
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_FGEN_MAX, &data.fgen_max);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_FGEN_MIN, &data.fgen_min);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_FSAMP_MAX, &data.fsamp_max);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_FSAMP_MIN, &data.fsamp_min);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_NDIV_MAX, &data.ndiv_max);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_NDIV_MIN, &data.ndiv_min);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_NREF_MAX, &data.nref_max);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_NREF_MIN, &data.nref_min);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_NVCO_MAX, &data.nvco_max);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_NVCO_MIN, &data.nvco_min);

	data.delta		= FLT_MAX;
	data.fref		= fref;
	data.fsamp_want	= LIMIT_RANGE(fsamp, data.fsamp_min, data.fsamp_max);
	data.fsamp_got	= LONG_MAX;
	data.delta		= LONG_MAX;
	data.mode		= 0;
	data.nvco		= 100;
	data.nref		= 100;
	data.ndiv		= 100;
	data.mca		= 0x8000;

	_dsi16wrc_fsamp_compute_pll(&data);

	mode[0]			= data.mode;
	nvco[0]			= data.nvco;
	nref[0]			= data.nref;
	ndiv[0]			= data.ndiv;
	mca[0]			= data.mca;

	if (rate)
		rate[0]	= data.fsamp_got;

	if (verbose)
	{
		printf("%s  (", errs ? " FAIL <---" : "");
		printf("Fref ");
		gsc_label_long_comma(data.fref);
		printf(", MCA 0x%04lX", (long) data.mca);
		psz	= (data.mode == DSI16WRC_ADC_MODE_HI_RES) ? "Hi Res" : "Hi Speed";
		printf(", %s", psz);
		printf(", Nvco %ld", (long) data.nvco);
		printf(", Nref %ld", (long) data.nref);
		printf(", Ndiv %ld", (long) data.ndiv);
		printf(", Fsamp %.3f", (double) data.fsamp_got);
		printf(")\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	dsi16wrc_fsamp_ai_report
*
*	Purpose:
*
*		Determine and report the sample rate for the specified channel.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		chan	The index of the channel of interest.
*
*		sps		Store the sample rate here, if non-NULL.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dsi16wrc_fsamp_ai_report(int fd, int index, int verbose, int chan, s32* sps)
{
	char	buf[64];
	int		errs		= 0;
	s32		fref;
	s32		fsamp		= 0;
	s32		g_src;
	s32		g0_src;
	s32		g1_src;
	s32		group;
	s32		mca;		// Master Clock Adjust
	s32		mode;
	s32		ndiv;
	s32		nref;
	s32		nvco;

	if (verbose)
	{
		sprintf(buf, "Channel %d", chan);
		gsc_label_index(buf, index);
	}

	gsc_label_level_inc();

	errs	+= dsi16wrc_channel_group	(fd, -1, 0, chan, &group);;
	errs	+= dsi16wrc_ch_grp_0_src	(fd, -1, 0, -1, &g0_src);
	errs	+= dsi16wrc_ch_grp_1_src	(fd, -1, 0, -1, &g1_src);
	errs	+= dsi16wrc_query			(fd, -1, 0, DSI16WRC_QUERY_FREF_DEFAULT, &fref);
	errs	+= dsi16wrc_master_clk_adj	(fd, -1, 0, -1, &mca);
	errs	+= dsi16wrc_adc_mode		(fd, -1, 0, -1, &mode);
	errs	+= dsi16wrc_ndiv			(fd, -1, 0, -1, &ndiv);
	errs	+= dsi16wrc_nref			(fd, -1, 0, -1, &nref);
	errs	+= dsi16wrc_nvco			(fd, -1, 0, -1, &nvco);

	g_src	= group ? g1_src : g0_src;
	errs	+= _fsamp_report_grp_src(verbose, &fsamp, g_src, fref, mca, mode, ndiv, nref, nvco);

	if (sps)
		sps[0]	= fsamp;

	gsc_label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	dsi16wrc_fsamp_ai_report_all
*
*	Purpose:
*
*		Determine and report the sample rate for all channels.
*
*	Arguments:
*
*		fd			The handle to use to access the driver.
*
*		index		The index of the device to access. Ignore if < 0.
*
*		verbose		Work verbosely?
*
*		fsamp		Report the overll sample rate here.
*
*	Returned:
*
*		>= 0		The number of errors encountered here.
*
******************************************************************************/

int dsi16wrc_fsamp_ai_report_all(int fd, int index, int verbose, s32* fsamp)
{
	s32	chans;
	int	chan;
	int	errs	= 0;
	s32	sps;
	s32	total	= 0;

	if (verbose)
	{
		gsc_label_index("Sample Rates", index);
		printf("\n");
	}

	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_CHANNEL_QTY, &chans);

	if (errs == 0)
	{
		if (verbose)
			gsc_label_level_inc();

		for (chan = 0; chan < chans; chan++)
		{
			errs	+= dsi16wrc_fsamp_ai_report(fd, -1, verbose, chan, &sps);
			total	+= sps;
		}

		if (verbose)
		{
			gsc_label("Overall Rate");
			gsc_label_long_comma(total);
			printf(" S/S");
			printf(" overall");
			printf("\n");
			gsc_label_level_dec();
		}
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	dsi16wrc_fsamp_ai_validate
*
*	Purpose:
*
*		Modify the target sample rate, if needed and report the results.
*
*	Arguments:
*
*		fd		The handle to use to access the driver.
*
*		index	The index of the device to access. Ignore if < 0.
*
*		verbose	Work verbosely?
*
*		fsamp	The desired sample rate is given here. We modifiy this if
*				needed.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi16wrc_fsamp_ai_validate(int fd, int index, int verbose, s32 fsamp)
{
	int	errs	= 0;
	s32	max;
	s32	min;

	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_FSAMP_MIN, &min);
	errs	+= dsi16wrc_query(fd, -1, 0, DSI16WRC_QUERY_FSAMP_MAX, &max);

	if (fsamp < min)
	{
		errs++;

		if (verbose)
		{
			printf("FAIL <---  (The minimum is ");
			gsc_label_long_comma((long) min);
			printf(" S/S.)\n");
		}
	}
	else if (fsamp > max)
	{
		errs++;

		if (verbose)
		{
			printf("FAIL <---  (The maximum is ");
			gsc_label_long_comma((long) max);
			printf(" S/S.)\n");
		}
	}

	return(errs);
}


