// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/utils/util_irq_sel.c $
// $Rev: 49274 $
// $Date: 2021-04-29 22:18:23 -0500 (Thu, 29 Apr 2021) $

// 24DSI16WRC: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi16wrc_irq_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI16WRC_IOCTL_IRQ_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi16wrc_irq_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ Select", index);

	ret		= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_IRQ_SEL, &set);
	errs	+= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case DSI16WRC_IRQ_INIT_DONE:

			ptr	= "Initialize Done";
			break;

		case DSI16WRC_IRQ_AUTO_CAL_DONE:

			ptr	= "Auto-Calibration Done";
			break;

		case DSI16WRC_IRQ_CHAN_READY:

			ptr	= "Channel Ready";
			break;

		case DSI16WRC_IRQ_AI_BUF_THRESH_L2H:

			ptr	= "AI Buffer Threshold Low-to-High";
			break;

		case DSI16WRC_IRQ_AI_BUF_THRESH_H2L:

			ptr	= "AI Buffer Threshold High-to-Low";
			break;

		case DSI16WRC_IRQ_AI_BURST_DONE:

			ptr	= "Burst Done";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


