// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/utils/util_query.c $
// $Rev: 52506 $
// $Date: 2023-03-01 12:36:56 -0600 (Wed, 01 Mar 2023) $

// 24DSI16WRC: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi16wrc_query
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI16WRC_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi16wrc_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;
	s32			tmp;

	switch (query)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Query Error (#%ld)", (long) query);
			break;

		case DSI16WRC_QUERY_AUTO_CAL_MS:	ptr	= "Auto-Cal Period";	break;
		case DSI16WRC_QUERY_CHANNEL_MAX:	ptr	= "Max Channels";		break;
		case DSI16WRC_QUERY_CHANNEL_QTY:	ptr	= "Input Channels";		break;
		case DSI16WRC_QUERY_COUNT:			ptr	= "Query Options";		break;
		case DSI16WRC_QUERY_DEVICE_TYPE:	ptr	= "Device Type";		break;
		case DSI16WRC_QUERY_EXT_SYNC_IN:	ptr	= "External SYNC In";	break;
		case DSI16WRC_QUERY_FGEN_MAX:		ptr	= "Fgen Max";			break;
		case DSI16WRC_QUERY_FGEN_MIN:		ptr	= "Fgen Min";			break;
		case DSI16WRC_QUERY_FIFO_SIZE:		ptr	= "FIFO Size";			break;
		case DSI16WRC_QUERY_FILTER_FREQ:	ptr	= "Filter Freq";		break;
		case DSI16WRC_QUERY_FREF_DEFAULT:	ptr	= "Fref Default";		break;
		case DSI16WRC_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";			break;
		case DSI16WRC_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";			break;
		case DSI16WRC_QUERY_INIT_MS:		ptr	= "Initialize Period";	break;
		case DSI16WRC_QUERY_NDIV_MAX:		ptr	= "Ndiv Max";			break;
		case DSI16WRC_QUERY_NDIV_MIN:		ptr	= "Ndiv Min";			break;
		case DSI16WRC_QUERY_NREF_MAX:		ptr	= "Nref Max";			break;
		case DSI16WRC_QUERY_NREF_MIN:		ptr	= "Nref Min";			break;
		case DSI16WRC_QUERY_NVCO_MAX:		ptr	= "Nvco Max";			break;
		case DSI16WRC_QUERY_NVCO_MIN:		ptr	= "Nvco Min";			break;
		case DSI16WRC_QUERY_V_RANGE:		ptr	= "Voltage Ranges";		break;
	}

	if (verbose)
		gsc_label_index(ptr, index);

	ret		= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_QUERY, &set);
	errs	+= ret ? 1 : 0;

	switch (query)
	{
		default:

			errs++;
			sprintf(buf, "%ld", (long) set);
			break;

		case DSI16WRC_QUERY_CHANNEL_MAX:
		case DSI16WRC_QUERY_CHANNEL_QTY:
		case DSI16WRC_QUERY_COUNT:
		case DSI16WRC_QUERY_NDIV_MAX:
		case DSI16WRC_QUERY_NDIV_MIN:
		case DSI16WRC_QUERY_NREF_MAX:
		case DSI16WRC_QUERY_NREF_MIN:
		case DSI16WRC_QUERY_NVCO_MAX:
		case DSI16WRC_QUERY_NVCO_MIN:

			gsc_label_long_comma_buf(set, buf);
			break;

		case DSI16WRC_QUERY_FGEN_MAX:
		case DSI16WRC_QUERY_FGEN_MIN:
		case DSI16WRC_QUERY_FILTER_FREQ:
		case DSI16WRC_QUERY_FREF_DEFAULT:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " Hz");
			break;

		case DSI16WRC_QUERY_AUTO_CAL_MS:
		case DSI16WRC_QUERY_INIT_MS:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " ms");
			break;

		case DSI16WRC_QUERY_FSAMP_MAX:
		case DSI16WRC_QUERY_FSAMP_MIN:

			gsc_label_long_comma_buf(set, buf);
			strcat(buf, " S/S");
			break;

		case DSI16WRC_QUERY_FIFO_SIZE:

			ptr	= "";
			tmp	= set;

			if ((tmp) && ((tmp % 1024) == 0))
			{
				ptr	= "K";
				tmp	/= 1024;
			}

			gsc_label_long_comma_buf(tmp, buf);
			strcat(buf, ptr);
			strcat(buf, " Words");
			break;

		case DSI16WRC_QUERY_DEVICE_TYPE:

			if (set == GSC_DEV_TYPE_24DSI16WRC)
			{
				strcpy(buf, "24DSI16WRC");
			}
			else
			{
				errs++;
				sprintf(buf, "INVALID: %ld", (long) set);
			}

			break;

		case DSI16WRC_QUERY_V_RANGE:

			if (set == DSI16WRC_QUERY_V_RANGE_10_BY_10)
			{
				strcpy(buf, "+-10V, +-1V, +-0.1V, +-0.01V");
			}
			else if (set == DSI16WRC_QUERY_V_RANGE_10_BY_2)
			{
				strcpy(buf, "+-10V, +-5V, +-2.5V, +-1.25V");
			}
			else
			{
				errs++;
				sprintf(buf, "UNKNOWN: %ld", (long) set);
			}

			break;

		case DSI16WRC_QUERY_EXT_SYNC_IN:

			strcpy(buf, set ? "Yes" : "No");
			break;
	}

	if (verbose)
	{
		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


