// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/24DSI16WRC/utils/util_reg_write.c $
// $Rev: 44343 $
// $Date: 2019-01-28 15:13:50 -0600 (Mon, 28 Jan 2019) $

// 24DSI16WRC: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dsi16wrc_reg_write
*
*	Purpose:
*
*		Provide a visual wrapper for the DSI16WRC_IOCTL_REG_WRITE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		reg		This is the register to access.
*
*		set		This is the value to apply.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int dsi16wrc_reg_write(int fd, int index, int verbose, u32 reg, u32 set)
{
	gsc_reg_t	arg;
	int			errs	= 0;
	int			ret;
	const char*	name;

	if (verbose)
		gsc_label_index("Register Write", index);

	arg.reg		= reg;
	arg.value	= set;
	arg.mask	= 0;	// unused here
	ret			= dsi16wrc_ioctl(fd, DSI16WRC_IOCTL_REG_WRITE, &arg);
	errs		+= ret ? 1 : 0;

	if (verbose)
	{
		name	= dsi16wrc_reg_get_name(reg);
		printf(	"%s  (val 0x%08lX, %s)\n",
				errs ? "FAIL <---" : "PASS",
				(unsigned long) arg.value,
				name ? name : "UNKNOWN");
	}

	return(errs);
}


